# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
import socket

from rekall import obj
from rekall import utils


AF_INET = 2
AF_INET6 = 0x17

# String representations of INADDR_ANY and INADDR6_ANY
inaddr_any = utils.inet_ntop(socket.AF_INET, '\0' * 4)
inaddr6_any = utils.inet_ntop(socket.AF_INET6, '\0' * 16)


# Structures used by connections, connscan, sockets, sockscan.
# Used by x86 XP (all service packs) and x86 2003 SP0.
tcpip_vtypes = {
    '_ADDRESS_OBJECT' : [ 0x68, {
    'Next' : [ 0x0, ['pointer', ['_ADDRESS_OBJECT']]],
    'LocalIpAddress' : [ 0x2c, ['IpAddress']],
    'LocalPort' : [ 0x30, ['unsigned be short']],
    'Protocol'  : [ 0x32, ['unsigned short']],
    'Pid' : [ 0x148, ['unsigned long']],
    'CreateTime' : [ 0x158, ['WinTimeStamp', {}]],
  }],
    '_TCPT_OBJECT' : [ 0x20, {
    'Next' : [ 0x0, ['pointer', ['_TCPT_OBJECT']]],
    'RemoteIpAddress' : [ 0xc, ['IpAddress']],
    'LocalIpAddress' : [ 0x10, ['IpAddress']],
    'RemotePort' : [ 0x14, ['unsigned be short']],
    'LocalPort' : [ 0x16, ['unsigned be short']],
    'Pid' : [ 0x18, ['unsigned long']],
    }],
}

# Structures used by connections, connscan, sockets, sockscan.
# Used by x64 XP and x64 2003 (all service packs).
tcpip_vtypes_2003_x64 = {
    '_ADDRESS_OBJECT' : [ 0x250, {
    'Next' : [ 0x0, ['pointer', ['_ADDRESS_OBJECT']]],
    'LocalIpAddress' : [ 0x58, ['IpAddress']],
    'LocalPort' : [ 0x5c, ['unsigned be short']],
    'Protocol'  : [ 0x5e, ['unsigned short']],
    'Pid' : [ 0x238, ['unsigned long']],
    'CreateTime' : [ 0x248, ['WinTimeStamp', {}]],
  }],
    '_TCPT_OBJECT' : [ 0x28, {
    'Next' : [ 0x0, ['pointer', ['_TCPT_OBJECT']]],
    'RemoteIpAddress' : [ 0x14, ['IpAddress']],
    'LocalIpAddress' : [ 0x18, ['IpAddress']],
    'RemotePort' : [ 0x1c, ['unsigned be short']],
    'LocalPort' : [ 0x1e, ['unsigned be short']],
    'Pid' : [ 0x20, ['unsigned long']],
    }],
}

# Structures used by sockets and sockscan.
# Used by x86 2003 SP1 and SP2 only.
tcpip_vtypes_2003_sp1_sp2 = {
    '_ADDRESS_OBJECT' : [ 0x68, {
    'Next' : [ 0x0, ['pointer', ['_ADDRESS_OBJECT']]],
    'LocalIpAddress' : [ 0x30, ['IpAddress']],
    'LocalPort' : [ 0x34, ['unsigned be short']],
    'Protocol'  : [ 0x36, ['unsigned short']],
    'Pid' : [ 0x14C, ['unsigned long']],
    'CreateTime' : [ 0x158, ['WinTimeStamp', {}]],
    }],
}

TCP_STATE_ENUM = {
    0: 'CLOSED', 1: 'LISTENING', 2: 'SYN_SENT',
    3: 'SYN_RCVD', 4: 'ESTABLISHED', 5: 'FIN_WAIT1',
    6: 'FIN_WAIT2', 7: 'CLOSE_WAIT', 8: 'CLOSING',
    9: 'LAST_ACK', 12: 'TIME_WAIT', 13: 'DELETE_TCB'
}

# Structures used by netscan for x86 Vista and 2008 (all service packs).
tcpip_vtypes_vista = {
    '_IN_ADDR' : [ None, {
    'addr4' : [ 0x0, ['IpAddress']],
    'addr6' : [ 0x0, ['Ipv6Address']],
    }],
    '_LOCAL_ADDRESS' : [ None, {
    'pData' : [ 0xC, ['pointer', ['pointer', ['_IN_ADDR']]]],
    }],
    '_TCP_LISTENER': [ 0xa8, { # TcpL
    'Owner' : [ 0x18, ['pointer', ['_EPROCESS']]],
    'CreateTime' : [ 0x20, ['WinTimeStamp', {}]],
    'LocalAddr' : [ 0x34, ['pointer', ['_LOCAL_ADDRESS']]],
    'InetAF' : [ 0x38, ['pointer', ['_INETAF']]],
    'Port' : [ 0x3E, ['unsigned be short']],
    }],
    '_TCP_ENDPOINT': [ 0x1f0, { # TcpE
    'InetAF' : [ 0xC, ['pointer', ['_INETAF']]],
    'AddrInfo' : [ 0x10, ['pointer', ['_ADDRINFO']]],
    'ListEntry': [ 0x14, ['_LIST_ENTRY']],
    'State' : [ 0x28, ['Enumeration', dict(target = 'long', choices = TCP_STATE_ENUM)]],
    'LocalPort' : [ 0x2C, ['unsigned be short']],
    'RemotePort' : [ 0x2E, ['unsigned be short']],
    'Owner' : [ 0x160, ['pointer', ['_EPROCESS']]],
    'CreateTime' : [ 0, ['WinTimeStamp', {}]],
    }],
    '_TCP_SYN_ENDPOINT': [ None, {
    'ListEntry': [ 8, ['_LIST_ENTRY']],
    'InetAF' : [ 0x18, ['pointer', ['_INETAF']]],
    'LocalPort' : [ 0x3c, ['unsigned be short']],
    'RemotePort' : [ 0x3e, ['unsigned be short']],
    'LocalAddr' : [ 0x1c, ['pointer', ['_LOCAL_ADDRESS']]],
    'RemoteAddress' : [ 0x28, ['pointer', ['_IN_ADDR']]],
    'Owner' : [ 0x20, ['pointer', ['_SYN_OWNER']]],
    'CreateTime' : [ 0, ['WinTimeStamp', {}]],
    }],
    '_SYN_OWNER': [ None, {
    'Process': [ 0x18, ['pointer', ['_EPROCESS']]],
    }],
    '_TCP_TIMEWAIT_ENDPOINT': [ None, {
    'ListEntry': [ 0x14, ['_LIST_ENTRY']],
    'InetAF' : [ 0xc, ['pointer', ['_INETAF']]],
    'LocalPort' : [ 0x1c, ['unsigned be short']],
    'RemotePort' : [ 0x1e, ['unsigned be short']],
    'LocalAddr' : [ 0x20, ['pointer', ['_LOCAL_ADDRESS']]],
    'RemoteAddress' : [ 0x24, ['pointer', ['_IN_ADDR']]],
    'CreateTime' : [ 0, ['WinTimeStamp', {}]],
    }],
    '_INETAF' : [ None, {
    'AddressFamily' : [ 0xC, ['unsigned short']],
    }],
    '_ADDRINFO' : [ None, {
    'Local' : [ 0x0, ['pointer', ['_LOCAL_ADDRESS']]],
    'Remote' : [ 0x8, ['pointer', ['_IN_ADDR']]],
    }],
    '_UDP_ENDPOINT': [ 0xa8, { # UdpA
    'Owner' : [ 0x18, ['pointer', ['_EPROCESS']]],
    'CreateTime' : [ 0x30, ['WinTimeStamp', {}]],
    'LocalAddr' : [ 0x38, ['pointer', ['_LOCAL_ADDRESS']]],
    'InetAF' : [ 0x14, ['pointer', ['_INETAF']]],
    'Port' : [ 0x48, ['unsigned be short']],
    }],
}

# Structures for netscan on x86 Windows 7 (all service packs).
tcpip_vtypes_7 = {
    '_TCP_ENDPOINT': [ 0x210, { # TcpE
    'InetAF' : [ 0xC, ['pointer', ['_INETAF']]],
    'AddrInfo' : [ 0x10, ['pointer', ['_ADDRINFO']]],
    'ListEntry': [ 0x14, ['_LIST_ENTRY']],
    'State' : [ 0x34, ['Enumeration', dict(target = 'long', choices = TCP_STATE_ENUM)]],
    'LocalPort' : [ 0x38, ['unsigned be short']],
    'RemotePort' : [ 0x3A, ['unsigned be short']],
    'Owner' : [ 0x174, ['pointer', ['_EPROCESS']]],
    'CreateTime' : [ 0, ['WinTimeStamp', {}]],
    }],
    '_TCP_SYN_ENDPOINT': [ None, {
    'ListEntry': [ 8, ['_LIST_ENTRY']],
    'InetAF' : [ 0x24, ['pointer', ['_INETAF']]],
    'LocalPort' : [ 0x48, ['unsigned be short']],
    'RemotePort' : [ 0x4a, ['unsigned be short']],
    'LocalAddr' : [ 0x28, ['pointer', ['_LOCAL_ADDRESS']]],
    'RemoteAddress' : [ 0x34, ['pointer', ['_IN_ADDR']]],
    'Owner' : [ 0x2c, ['pointer', ['_SYN_OWNER']]],
    'CreateTime' : [ 0, ['WinTimeStamp', {}]],
    }],
    '_TCP_TIMEWAIT_ENDPOINT': [ None, {
    'ListEntry': [ 0, ['_LIST_ENTRY']],
    'InetAF' : [ 0x18, ['pointer', ['_INETAF']]],
    'LocalPort' : [ 0x28, ['unsigned be short']],
    'RemotePort' : [ 0x2a, ['unsigned be short']],
    'LocalAddr' : [ 0x2c, ['pointer', ['_LOCAL_ADDRESS']]],
    'RemoteAddress' : [ 0x30, ['pointer', ['_IN_ADDR']]],
    'CreateTime' : [ 0, ['WinTimeStamp', {}]],
    }],
}

# Structures for netscan on x64 Vista SP0 and 2008 SP0
tcpip_vtypes_vista_64 = {
    '_IN_ADDR' : [ None, {
    'addr4' : [ 0x0, ['IpAddress']],
    'addr6' : [ 0x0, ['Ipv6Address']],
    }],
    '_TCP_LISTENER': [ 0x120, { # TcpL
    'Owner' : [ 0x28, ['pointer', ['_EPROCESS']]],
    'CreateTime' : [ 0x20, ['WinTimeStamp', {}]],
    'LocalAddr' : [ 0x58, ['pointer', ['_LOCAL_ADDRESS']]],
    'InetAF' : [ 0x60, ['pointer', ['_INETAF']]],
    'Port' : [ 0x6a, ['unsigned be short']],
    }],
    '_INETAF' : [ None, {
    'AddressFamily' : [ 0x14, ['unsigned short']],
    }],
    '_LOCAL_ADDRESS' : [ None, {
    'pData' : [ 0x10, ['pointer', ['pointer', ['_IN_ADDR']]]],
    }],
    '_ADDRINFO' : [ None, {
    'Local' : [ 0x0, ['pointer', ['_LOCAL_ADDRESS']]],
    'Remote' : [ 0x10, ['pointer', ['_IN_ADDR']]],
    }],
    '_TCP_ENDPOINT': [ 0x210, { # TcpE
    'InetAF' : [ 0x18, ['pointer', ['_INETAF']]],
    'AddrInfo' : [ 0x20, ['pointer', ['_ADDRINFO']]],
    'ListEntry': [ 0x28, ['_LIST_ENTRY']],
    'State' : [ 0x50, ['Enumeration', dict(target = 'long', choices = TCP_STATE_ENUM)]],
    'LocalPort' : [ 0x54, ['unsigned be short']],
    'RemotePort' : [ 0x56, ['unsigned be short']],
    'Owner' : [ 0x208, ['pointer', ['_EPROCESS']]],
    'CreateTime' : [ 0, ['WinTimeStamp', {}]],
    }],
    '_TCP_SYN_ENDPOINT': [ None, {
    'ListEntry': [ 0x10, ['_LIST_ENTRY']],
    'InetAF' : [ 0x30, ['pointer', ['_INETAF']]],
    'LocalPort' : [ 0x64, ['unsigned be short']],
    'RemotePort' : [ 0x66, ['unsigned be short']],
    'LocalAddr' : [ 0x38, ['pointer', ['_LOCAL_ADDRESS']]],
    'RemoteAddress' : [ 0x50, ['pointer', ['_IN_ADDR']]],
    'Owner' : [ 0x40, ['pointer', ['_SYN_OWNER']]],
    'CreateTime' : [ 0, ['WinTimeStamp', {}]],
    }],
    '_SYN_OWNER': [ None, {
    'Process': [ 0x28, ['pointer', ['_EPROCESS']]],
    }],
    '_TCP_TIMEWAIT_ENDPOINT': [ None, {
    'ListEntry': [ 0, ['_LIST_ENTRY']],
    'InetAF' : [ 0x18, ['pointer', ['_INETAF']]],
    'LocalPort' : [ 0x30, ['unsigned be short']],
    'RemotePort' : [ 0x32, ['unsigned be short']],
    'LocalAddr' : [ 0x38, ['pointer', ['_LOCAL_ADDRESS']]],
    'RemoteAddress' : [ 0x40, ['pointer', ['_IN_ADDR']]],
    'CreateTime' : [ 0, ['WinTimeStamp', {}]],
    }],
    '_UDP_ENDPOINT': [ 0x82, { # UdpA
    'Owner' : [ 0x28, ['pointer', ['_EPROCESS']]],
    'CreateTime' : [ 0x58, ['WinTimeStamp', {}]],
    'LocalAddr' : [ 0x60, ['pointer', ['_LOCAL_ADDRESS']]],
    'InetAF' : [ 0x20, ['pointer', ['_INETAF']]],
    'Port' : [ 0x80, ['unsigned be short']],
    }],
}


tcpip_vtypes_win7_64 = {
    '_TCP_ENDPOINT': [ 0x320, {
            'State' : [ 0x68, ['Enumeration', dict(target='long',
                                                   choices=TCP_STATE_ENUM)]],

            'LocalPort' : [ 0x6c, ['unsigned be short']],
            'RemotePort' : [ 0x6e, ['unsigned be short']],
            'Owner' : [ 0x238, ['pointer', ['_EPROCESS']]],
            }],
    '_TCP_SYN_ENDPOINT': [ None, {
            'InetAF' : [ 0x48, ['pointer', ['_INETAF']]],
            'LocalPort' : [ 0x7c, ['unsigned be short']],
            'RemotePort' : [ 0x7e, ['unsigned be short']],
            'LocalAddr' : [ 0x50, ['pointer', ['_LOCAL_ADDRESS']]],
            'RemoteAddress' : [ 0x68, ['pointer', ['_IN_ADDR']]],
            'Owner' : [ 0x58, ['pointer', ['_SYN_OWNER']]],
            }],
    '_TCP_TIMEWAIT_ENDPOINT': [ None, {
            'InetAF' : [ 0x30, ['pointer', ['_INETAF']]],
            'LocalPort' : [ 0x48, ['unsigned be short']],
            'RemotePort' : [ 0x4a, ['unsigned be short']],
            'LocalAddr' : [ 0x50, ['pointer', ['_LOCAL_ADDRESS']]],
            'RemoteAddress' : [ 0x58, ['pointer', ['_IN_ADDR']]],
            }],
    }


#--------------------------------------------------------------------------------
# object classes
#--------------------------------------------------------------------------------

class _TCP_LISTENER(obj.Struct):
    """Class for objects found in TcpL pools"""

    def dual_stack_sockets(self, vm=None):
        """Handle Windows dual-stack sockets"""

        # If this pointer is valid, the socket is bound to
        # a specific IP address. Otherwise, the socket is
        # listening on all IP addresses of the address family.
        local_addr = self.LocalAddr.dereference(vm=vm)

        # Switch to the correct address space.
        af_inet = self.InetAF.dereference(vm=vm)

        # Note the remote address is always INADDR_ANY or
        # INADDR6_ANY for sockets. The moment a client
        # connects to the listener, a TCP_ENDPOINT is created
        # and that structure contains the remote address.
        if local_addr:
            inaddr = local_addr.pData.dereference().dereference()
            if af_inet.AddressFamily == AF_INET:
                yield "v4", inaddr.addr4, inaddr_any
            else:
                yield "v6", inaddr.addr6, inaddr6_any
        else:
            yield "v4", inaddr_any, inaddr_any
            if af_inet.AddressFamily.v() == AF_INET6:
                yield "v6", inaddr6_any, inaddr6_any


class _TCP_ENDPOINT(_TCP_LISTENER):
    """Class for objects found in TcpE pools"""

    def _ipv4_or_ipv6(self, in_addr, vm=None):
        af_inet = self.InetAF.dereference(vm=vm)
        if af_inet.AddressFamily == AF_INET:
            return in_addr.addr4
        else:
            return in_addr.addr6

    def LocalAddress(self, vm=None):
        inaddr = self.AddrInfo.dereference(vm=vm).Local.\
            pData.dereference().dereference()

        return self._ipv4_or_ipv6(inaddr, vm=vm)

    def RemoteAddress(self, vm=None):
        inaddr = self.AddrInfo.dereference(vm=vm).\
            Remote.dereference()

        return self._ipv4_or_ipv6(inaddr, vm=vm)


class _UDP_ENDPOINT(_TCP_LISTENER):
    """Class for objects found in UdpA pools"""


class TCPIPModifications(obj.ProfileModification):
    """A profile modification which adds structures related to TCP sockets.

    Note that most of these do not come from the PDB files, but are reversed.
    """

    @classmethod
    def modify(cls, profile):
        # Network Object Classess for Vista, 2008, and 7 x86 and x64
        if profile.metadatas("major", "minor") >= (6, 0):
            profile.add_classes(dict(_TCP_LISTENER=_TCP_LISTENER,
                                     _TCP_ENDPOINT=_TCP_ENDPOINT,
                                     _UDP_ENDPOINT=_UDP_ENDPOINT))

        if profile.metadata("memory_model") == "64bit":
            # Vista SP1.
            if (profile.metadatas("major", "minor") == (6, 0) and
                profile.metadata("build") >= 6001):
                profile.add_overlay(tcpip_vtypes_vista_64)
                profile.add_overlay({
                        '_TCP_ENDPOINT': [ None, {
                                'Owner' : [ 0x210, ['pointer', ['_EPROCESS']]],
                                }],
                        })

            # Windows 7
            elif profile.metadatas("major", "minor") == (6, 1):
                profile.add_overlay(tcpip_vtypes_vista_64)
                profile.add_overlay(tcpip_vtypes_win7_64)

            # Win2k3
            elif profile.metadatas("major", "minor") == (5, 2):
                profile.add_overlay(tcpip_vtypes_2003_x64)

        elif profile.metadata("memory_model") == "32bit":
            profile.add_overlay(tcpip_vtypes)

            # Win2k3
            if profile.metadatas("major", "minor") == (5, 2):
                profile.add_overlay(tcpip_vtypes_2003_sp1_sp2)

            # Vista
            elif profile.metadatas("major", "minor") == (6, 0):
                profile.add_overlay(tcpip_vtypes_vista)

            # Windows 7
            elif profile.metadatas("major", "minor") == (6, 1):
                profile.add_overlay(tcpip_vtypes_7)

        # Pool tags
        profile.add_constants(UDP_END_POINT_POOLTAG="UdpA",
                              TCP_LISTENER_POOLTAG="TcpL",
                              TCP_END_POINT_POOLTAG="TcpE")

        return profile



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

__author__ = "Michael Cohen <scudette@google.com>"

from rekall.plugins.linux import common
from rekall.plugins.overlays import basic


class Netstat(common.LinuxPlugin):
    """Print the active network connections."""

    __name = "netstat"


    def sockets(self):
        """Enumerate all socket objects."""

        lsof = self.session.plugins.lsof(session=self.session)
        for task, file_struct, fd in lsof.lsof():
            if (file_struct.f_op == self.profile.get_constant(
                    "socket_file_ops") or
                file_struct.m("d_entry").d_op == self.profile.get_constant(
                    "sockfs_dentry_operations")):

                iaddr = file_struct.dentry.d_inode

                # See http://lxr.free-electrons.com/source/include/net/sock.h?v=3.8#L1319
                skt = basic.container_of(iaddr, "socket_alloc",
                                         "vfs_inode").socket

                yield task, fd, skt.sk, iaddr

    def render(self, renderer):
        unix_sockets = []
        tcp_sockets = []

        for task, fd, sock, iaddr in self.sockets():
            inet_sock = sock.dereference_as("inet_sock")

            if sock.sk_protocol not in ("IPPROTO_TCP", "IPPROTO_UDP", "IPPROTO_IPV4", "IPPROTO_IPV6", "IPPROTO_HOPOPT"):
                continue

            sk_common = sock.m("__sk_common")

            if sk_common.skc_family == "AF_UNIX":
                unix_sock = sock.dereference_as("unix_sock")
                name = unix_sock.addr.name[0].sun_path
                unix_sockets.append((task, fd, sock, iaddr, sk_common))

            elif sk_common.skc_family in ("AF_INET", "AF_INET6"):
                tcp_sockets.append((task, fd, sock, iaddr, sk_common))


        # First do the tcp sockets.
        renderer.table_header([("Proto", "proto", "8"),
                               ("SAddr", "saddr", "15"),
                               ("SPort", "sport", "8"),
                               ("DAddr", "daddr", "15"),
                               ("DPort", "dport", "5"),
                               ("State", "state", "15"),
                               ("Pid", "pid", "8"),
                               ("Comm", "comm", "20")])

        for task, fd, sock, iaddr, sk_common in tcp_sockets:
            inet_sock = sock.dereference_as("inet_sock")

            renderer.table_row(
                sk_common.skc_family,
                inet_sock.src_addr,
                inet_sock.src_port,
                inet_sock.dst_addr,
                inet_sock.dst_port,
                sk_common.skc_state,
                task.pid,
                task.comm,
                )

        # Now do the udp sockets.
        renderer.table_header([("Proto", "proto", "8"),
                               ("Ref Count", "ref", "^6"),
                               ("Type", "type", "12"),
                               ("State", "state", "18"),
                               ("Inode", "inode", "8"),
                               ("Path", "path", "20")])

        for task, fd, sock, iaddr, sk_common in unix_sockets:
            unix_sock = sock.dereference_as("unix_sock")
            name = unix_sock.addr.name[0].sun_path

            renderer.table_row(
                sk_common.skc_family,
                unix_sock.addr.refcnt.counter,
                sock.sk_type,
                sk_common.skc_state,
                iaddr.i_ino,
                name
                )



# Rekall Memory Forensics
# Copyright (C) 2007,2008 Volatile Systems
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from rekall import obj
from rekall.plugins.windows.gui import constants


class XP2003x86BaseVTypes(obj.ProfileModification):
    """Applies to everything x86 before Windows 7"""

    @classmethod
    def modify(cls, profile):
        if profile.metadata("memory_model") == "32bit":
            profile.add_overlay(cls.vtypes_xp_32)
        else:
            profile.add_overlay(cls.vtypes_xp_64)

        return profile

    vtypes_xp_32 = {
        'tagWINDOWSTATION' : [ 0x5C, {
        'dwSessionId' : [ 0x0, ['unsigned long']],
        'rpwinstaNext' : [ 0x4, ['pointer', ['tagWINDOWSTATION']]],
        'rpdeskList' : [ 0x8, ['pointer', ['tagDESKTOP']]],
        'dwWSF_Flags' : [ 0x10, ['unsigned long']],
        'ptiDrawingClipboard' : [ 0x1C, ['pointer', ['tagTHREADINFO']]],
        'spwndClipOpen' : [ 0x20, ['pointer', ['tagWND']]],
        'spwndClipViewer' : [ 0x24, ['pointer', ['tagWND']]],
        'spwndClipOwner' : [ 0x28, ['pointer', ['tagWND']]],
        'pClipBase' : [ 0x2C, ['pointer', ['array', lambda x : x.cNumClipFormats, ['tagCLIP']]]],
        'cNumClipFormats' : [ 0x30, ['unsigned int']],
        'iClipSerialNumber' : [ 0x34, ['unsigned int']],
        'iClipSequenceNumber' : [ 0x38, ['unsigned int']],
        #'spwndClipboardListener' : [ 0x3C, ['pointer', ['tagWND']]],
        'pGlobalAtomTable' : [ 0x40, ['pointer', ['void']]],
        }],
        ## This is defined in Windows 7
        'tagCLIP' : [ 12, {
        'fmt' : [ 0, ['Enumeration', dict(target = 'unsigned long', choices = constants.CLIPBOARD_FORMAT_ENUM)]],
        'hData' : [ 4, ['unsigned int']],
        'fGlobalHandle' : [ 8, ['unsigned int']],
        }],
        'tagDESKTOP' : [ 0x84, {
        'dwSessionId' : [ 0x0, ['unsigned long']],
        'pDeskInfo' : [ 0x4, ['pointer', ['tagDESKTOPINFO']]],
        'rpdeskNext' : [ 0xc, ['pointer', ['tagDESKTOP']]],
        'rpwinstaParent' : [ 0x10, ['pointer', ['tagWINDOWSTATION']]],
        'hsectionDesktop' : [ 0x40, ['pointer', ['void']]],
        'pheapDesktop' : [ 0x44, ['pointer', ['tagWIN32HEAP']]],
        'PtiList' : [ 0x64, ['_LIST_ENTRY']],
        }],
        'tagTHREADINFO' : [ None, { # Same as Win32Thread
        'pEThread' : [ 0x00, ['pointer', ['_ETHREAD']]],
        'ppi' : [ 0x2C, ['pointer', ['tagPROCESSINFO']]],
        'pq' : [ 0x30, ['pointer', ['tagQ']]],
        'pDeskInfo' : [ 0x40, ['pointer', ['tagDESKTOPINFO']]],
        'PtiLink' : [ 0xAC, ['_LIST_ENTRY']],
        'fsHooks' : [ 0x98, ['unsigned long']],
        'aphkStart' : [ 0xF4, ['array', 16, ['pointer', ['tagHOOK']]]],
        }],
        'tagQ' : [ None, {
        'mlInput' : [ 0x00, ['tagMLIST']],
        }],
        'tagMLIST' : [ None, {
        'pqmsgRead' : [ 0x00, ['pointer', ['tagQMSG']]],
        'cMsgs' : [ 0x08, ['unsigned long']],
        }],
        'tagQMSG' : [ None, {
        'pqmsgNext' : [ 0x00, ['pointer', ['tagQMSG']]],
        'pqmsgPrev' : [ 0x04, ['pointer', ['tagQMSG']]],
        'msg' : [ 0x08, ['tagMSG']],
        }],
        'tagMSG' : [ None, {
        'hwnd' : [ 0x00, ['unsigned long']],
        'message' : [ 0x04, ['unsigned long']],
        'wParam' : [ 0x08, ['unsigned long']],
        'lParam' : [ 0x0C, ['unsigned long']],
        'time' : [ 0x10, ['unsigned long']],
        'pt' : [ 0x14, ['tagPOINT']],
        }],
        'tagPOINT' : [ None, {
        'x' : [ 0x00, ['long']],
        'y' : [ 0x04, ['long']],
        }],
        'tagHOOK' : [ None, {
        'head' : [ 0x0, ['_THRDESKHEAD']],
        'phkNext' : [ 0x14, ['pointer', ['tagHOOK']]],
        'iHook' : [ 0x18, ['long']],
        'offPfn' : [ 0x1c, ['unsigned long']],
        'flags': [ 0x20, ['Flags', {'bitmap': constants.HOOK_FLAGS}]],
        'ihmod' : [ 0x24, ['long']],
        'ptiHooked' : [ 0x28, ['pointer', ['tagTHREADINFO']]],
        'rpdesk' : [ 0x2c, ['pointer', ['tagDESKTOP']]],
        }],
        'tagDESKTOPINFO' : [ None, {
        'pvDesktopBase' : [ 0x0, ['pointer', ['void']]],
        'pvDesktopLimit' : [ 0x4, ['pointer', ['void']]],
        'spwnd' : [ 0x08, ['pointer', ['tagWND']]],
        'fsHooks' : [ 0x0c, ['unsigned long']],
        'aphkStart' : [ 0x10, ['array', 16, ['pointer', ['tagHOOK']]]],
        }],
        'tagSERVERINFO' : [ 0xffc, {
        'cHandleEntries' : [ 8, ['unsigned long']],
        'cbHandleTable' : [ 0x1bc, ['unsigned long']],
        }],
        'tagSHAREDINFO' : [ 0x11c, { # From Win7SP0x86
        'psi' : [ 0x0, ['pointer', ['tagSERVERINFO']]],
        'aheList' : [ 0x4, ['pointer', ['_HANDLEENTRY']]],
        'ulSharedDelta' : [ 0xC, ['unsigned long']],
        }],
        '_HANDLEENTRY' : [ 0xc, { # From Win7SP0x86
        'phead' : [ 0x0, ['pointer', ['_HEAD']]],
        'pOwner' : [ 0x4, ['pointer', ['void']]],
        'bType': [ 8, ['Enumeration', dict(target = 'unsigned char', choices = constants.HANDLE_TYPE_ENUM)]],
        'bFlags' : [ 0x9, ['unsigned char']],
        'wUniq' : [ 0xa, ['unsigned short']],
        }],
        '_HEAD' : [ 0x8, { # From Win7SP0x86
        'h' : [ 0x0, ['pointer', ['void']]],
        'cLockObj' : [ 0x4, ['unsigned long']],
        }],
        'tagPROCESSINFO' : [ None, {
        'Process' : [ 0x0, ['pointer', ['_EPROCESS']]],
        }],
        '_THRDESKHEAD' : [ 0x14, {
        'h' : [ 0x0, ['pointer', ['void']]],
        'cLockObj' : [ 0x4, ['unsigned long']],
        'pti' : [ 0x8, ['pointer', ['tagTHREADINFO']]],
        'rpdesk' : [ 0xc, ['pointer', ['tagDESKTOP']]],
        'pSelf' : [ 0x10, ['pointer', ['unsigned char']]],
        }],
        'tagCLS' : [ 0x5c, {
        'pclsNext' : [ 0x0, ['pointer', ['tagCLS']]],
        'atomClassName' : [ 0x4, ['unsigned short']],
        'atomNVClassName' : [ 0x6, ['unsigned short']],
        }],
        'tagRECT' : [ 0x10, {
        'left' : [ 0x0, ['long']],
        'top' : [ 0x4, ['long']],
        'right' : [ 0x8, ['long']],
        'bottom' : [ 0xc, ['long']],
        }],
        'tagWND' : [ 0x90, {
        'head' : [ 0x0, ['_THRDESKHEAD']],
        'ExStyle' : [ 0x1c, ['unsigned long']],
        'style' : [ 0x20, ['unsigned long']],
        'hModule' : [ 0x24, ['pointer', ['void']]],
        'spwndNext' : [ 0x2c, ['pointer', ['tagWND']]],
        'spwndPrev' : [ 0x30, ['pointer', ['tagWND']]],
        'spwndParent' : [ 0x34, ['pointer', ['tagWND']]],
        'spwndChild' : [ 0x38, ['pointer', ['tagWND']]],
        'spwndOwner' : [ 0x3c, ['pointer', ['tagWND']]],
        'rcWindow' : [ 0x40, ['tagRECT']],
        'rcClient' : [ 0x50, ['tagRECT']],
        'lpfnWndProc' : [ 0x60, ['pointer', ['void']]],
        'pcls' : [ 0x64, ['pointer', ['tagCLS']]],
        'strName' : [ 0x80, ['_LARGE_UNICODE_STRING']],
        'cbwndExtra' : [ 0x8C, ['long']],
        'dwUserData' : [ 0x98, ['unsigned long']],
        }],
        '_LARGE_UNICODE_STRING' : [ 0xc, {
        'Length' : [ 0x0, ['unsigned long']],
        'MaximumLength' : [ 0x4, ['BitField', dict(start_bit = 0, end_bit = 31)]],
        'bAnsi' : [ 0x4, ['BitField', dict(start_bit = 31, end_bit = 32)]],
        'Buffer' : [ 0x8, ['pointer', ['unsigned short']]],
        }],
    }

    vtypes_xp_64 = {
        'tagWINDOWSTATION' : [ 0x90, { # !poolfind Wind is 100h
        'dwSessionId' : [ 0x0, ['unsigned long']],
        'rpwinstaNext' : [ 0x8, ['pointer64', ['tagWINDOWSTATION']]], # FreeWindowStation
        'rpdeskList' : [ 0x10, ['pointer64', ['tagDESKTOP']]],
        'dwWSF_Flags' : [ 0x20, ['unsigned long']], # FreeWindowStation
        'ptiDrawingClipboard' : [ 0x38, ['pointer64', ['tagTHREADINFO']]], # xxxDrawClipboard
        'spwndClipOpen' : [ 0x40, ['pointer64', ['tagWND']]],
        'spwndClipViewer' : [ 0x48, ['pointer64', ['tagWND']]],
        'spwndClipOwner' : [ 0x50, ['pointer64', ['tagWND']]],
        'pClipBase' : [ 0x58, ['pointer64', ['array', lambda x : x.cNumClipFormats, ['tagCLIP']]]], # InternalSetClipboardData
        'cNumClipFormats' : [ 0x60, ['unsigned int']], # InternalSetClipboardData
        'iClipSerialNumber' : [ 0x64, ['unsigned int']], # InternalSetClipboardData
        'iClipSequenceNumber' : [ 0x68, ['unsigned int']], # InternalSetClipboardData
        'pGlobalAtomTable' : [ 0x70, ['pointer64', ['void']]],
        }],

        # From Windows 7
        'tagCLIP' : [ 0x18, {
        'fmt' : [ 0x0, ['Enumeration', dict(target = 'unsigned long', choices = constants.CLIPBOARD_FORMAT_ENUM)]],
        'hData' : [ 0x8, ['pointer64', ['void']]],
        'fGlobalHandle' : [ 0x10, ['long']],
        }],

        'tagDESKTOP' : [ 0xd0, { # !poolfind Desk is 140h
        'dwSessionId' : [ 0x0, ['unsigned long']],
        'pDeskInfo' : [ 0x8, ['pointer64', ['tagDESKTOPINFO']]], # xxxCreateDesktop
        'rpdeskNext' : [ 0x18, ['pointer64', ['tagDESKTOP']]], # ParseDesktop
        'rpwinstaParent' : [ 0x20, ['pointer64', ['tagWINDOWSTATION']]],
        'hsectionDesktop' : [ 0x70, ['pointer64', ['void']]], # MapDesktop
        'pheapDesktop' : [ 0x78, ['pointer64', ['tagWIN32HEAP']]], # DesktopAlloc
        'PtiList' : [ 0xa0, ['_LIST_ENTRY']], # zzzJournalAttach
        }],

        'tagTHREADINFO' : [ None, {
        'pEThread' : [ 0x00, ['pointer', ['_ETHREAD']]],
        'ppi' : [ 0x68, ['pointer64', ['tagPROCESSINFO']]], # xxxSetThreadDesktop
        #'pq' : [ 0x30, ['pointer', ['tagQ']]],
        'pDeskInfo' : [ 0x90, ['pointer64', ['tagDESKTOPINFO']]], # xxxDesktopThread
        'PtiLink' : [ 0x160, ['_LIST_ENTRY']],
        'fsHooks' : [ 0x138, ['unsigned long']], # xxxSetThreadDesktop, CheckWHFBits
        'aphkStart' : [ 0x140, ['array', 16, ['pointer64', ['tagHOOK']]]],
        }],

        'tagDESKTOPINFO' : [ None, {
        'pvDesktopBase' : [ 0x0, ['pointer64', ['void']]],
        'pvDesktopLimit' : [ 0x8, ['pointer64', ['void']]],
        'spwnd' : [ 0x10, ['pointer64', ['tagWND']]],
        'fsHooks' : [ 0x18, ['unsigned long']], # CheckWHFBits
        'aphkStart' : [ 0x20, ['array', 16, ['pointer64', ['tagHOOK']]]],
        }],

        'tagWND' : [ None, {
        'head' : [ 0x0, ['_THRDESKHEAD']],
        'ExStyle' : [ 0x30, ['unsigned long']], # xxxCreateWindowEx
        'style' : [ 0x34, ['unsigned long']], # xxxCreateWindowEx
        'spwndNext' : [ 0x48, ['pointer64', ['tagWND']]],
        'spwndPrev' : [ 0x50, ['pointer64', ['tagWND']]],
        'spwndParent' : [ 0x58, ['pointer64', ['tagWND']]],
        'spwndChild' : [ 0x60, ['pointer64', ['tagWND']]],
        'spwndOwner' : [ 0x68, ['pointer64', ['tagWND']]],
        'rcWindow' : [ 0x70, ['tagRECT']],
        'rcClient' : [ 0x80, ['tagRECT']],
        'lpfnWndProc' : [ 0x90, ['pointer64', ['void']]],
        'pcls' : [ 0x98, ['pointer64', ['tagCLS']]], # HMChangeOwnerThread
        'strName' : [ 0xd0, ['_LARGE_UNICODE_STRING']],
        }],

        'tagRECT' : [ 0x10, {
        'left' : [ 0x0, ['long']],
        'top' : [ 0x4, ['long']],
        'right' : [ 0x8, ['long']],
        'bottom' : [ 0xc, ['long']],
        }],

        'tagCLS' : [ None, {
        'pclsNext' : [ 0x0, ['pointer64', ['tagCLS']]],
        'atomClassName' : [ 0x8, ['unsigned short']], # HMChangeOwnerThread
        'atomNVClassName' : [ 0xA, ['unsigned short']],
        }],

        # From Win7 x64
        '_LARGE_UNICODE_STRING' : [ 0x10, {
        'Length' : [ 0x0, ['unsigned long']],
        'MaximumLength' : [ 0x4, ['BitField', dict(start_bit = 0, end_bit = 31, native_type = 'unsigned long')]],
        'bAnsi' : [ 0x4, ['BitField', dict(start_bit = 31, end_bit = 32, native_type = 'unsigned long')]],
        'Buffer' : [ 0x8, ['pointer64', ['unsigned short']]],
        }],

        # From Win7 x64
        '_THRDESKHEAD' : [ 0x28, {
        'h' : [ 0x0, ['pointer64', ['void']]],
        'cLockObj' : [ 0x8, ['unsigned long']],
        'pti' : [ 0x10, ['pointer64', ['tagTHREADINFO']]],
        'rpdesk' : [ 0x18, ['pointer64', ['tagDESKTOP']]],
        'pSelf' : [ 0x20, ['pointer64', ['unsigned char']]],
        }],

        # From Win7 x64
        'tagSHAREDINFO' : [ None, {
        'psi' : [ 0x0, ['pointer64', ['tagSERVERINFO']]],
        'aheList' : [ 0x8, ['pointer64', ['_HANDLEENTRY']]],
        #'HeEntrySize' : [ 0x10, ['unsigned long']],
        #'pDispInfo' : [ 0x18, ['pointer64', ['tagDISPLAYINFO']]],
        'ulSharedDelta' : [ 0x18, ['unsigned long long']],
        #'awmControl' : [ 0x28, ['array', 31, ['_WNDMSG']]],
        #'DefWindowMsgs' : [ 0x218, ['_WNDMSG']],
        #'DefWindowSpecMsgs' : [ 0x228, ['_WNDMSG']],
        }],

        # From Win7 x64
        '_HANDLEENTRY' : [ 0x18, {
        'phead' : [ 0x0, ['pointer64', ['_HEAD']]],
        'pOwner' : [ 0x8, ['pointer64', ['void']]],
        'bType': [ 0x10, ['Enumeration', dict(target = 'unsigned char', choices = constants.HANDLE_TYPE_ENUM)]],
        'bFlags' : [ 0x11, ['unsigned char']],
        'wUniq' : [ 0x12, ['unsigned short']],
        }],

        # From Win7 x64
        '_HEAD' : [ 0x10, {
        'h' : [ 0x0, ['pointer64', ['void']]],
        'cLockObj' : [ 0x8, ['unsigned long']],
        }],

        'tagSERVERINFO' : [ None, {
        'cHandleEntries' : [ 8, ['unsigned long']],
        'cbHandleTable' : [ 0x330, ['unsigned long']], # HMInitHandleTable
        }],

        'tagPROCESSINFO' : [ None, {
        'Process' : [ 0x0, ['pointer', ['_EPROCESS']]],
        }],

        # From Win7 x64
        'tagHOOK' : [ 0x60, {
        'head' : [ 0x0, ['_THRDESKHEAD']],
        'phkNext' : [ 0x28, ['pointer64', ['tagHOOK']]],
        'iHook' : [ 0x30, ['long']],
        'offPfn' : [ 0x38, ['unsigned long long']],
        'flags': [ 0x40, ['Flags', {'bitmap': constants.HOOK_FLAGS}]],
        'ihmod' : [ 0x44, ['long']],
        'ptiHooked' : [ 0x48, ['pointer64', ['tagTHREADINFO']]],
        'rpdesk' : [ 0x50, ['pointer64', ['tagDESKTOP']]],
        'nTimeout' : [ 0x58, ['BitField', dict(start_bit = 0, end_bit = 7, native_type = 'unsigned long')]],
        'fLastHookHung' : [ 0x58, ['BitField', dict(start_bit = 7, end_bit = 8, native_type = 'long')]],
        }],
    }




# Rekall Memory Forensics
# Copyright (c) 2012, Michael Cohen <scudette@gmail.com>
# Copyright (c) 2010, 2011, 2012 Michael Ligh <michael.ligh@mnin.org>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""A Rekall Memory Forensics scanner which uses yara."""
import sys
import yara

from rekall import constants
from rekall import plugin
from rekall import scan
from rekall import utils
from rekall.plugins.windows import common
from rekall.plugins.windows import vadinfo


class BaseYaraASScanner(scan.BaseScanner):
    """An address space scanner for Yara signatures."""
    overlap = 1024

    def __init__(self, rules=None, **kwargs):
        super(BaseYaraASScanner, self).__init__(**kwargs)
        self.rules = rules

    def scan(self, offset=0, maxlen=None):
        maxlen = maxlen or self.profile.get_constant("MaxPointer")

        # Start scanning from offset until maxlen:
        i = offset

        while i < offset + maxlen:
            # Update the progress bar.
            if self.session:
                self.session.report_progress("Yara Scanning %s (0x%08X)" % (
                        self.address_space, i))

            # Read some data and match it.
            to_read = min(constants.SCAN_BLOCKSIZE, maxlen - (i - offset))
            data = self.address_space.read(i, to_read)
            if data:
                for match in self.rules.match(data=data):
                    for offset, name, value in match.strings:
                        if offset < constants.SCAN_BLOCKSIZE - self.overlap:
                            yield match.rule, offset + i, name, value

            i += constants.SCAN_BLOCKSIZE - self.overlap


class DiscontigYaraScanner(scan.DiscontigScanner, BaseYaraASScanner):
    """A discontiguous scanner based on yara signatures."""


class VadYarraScanner(vadinfo.VadScanner, BaseYaraASScanner):
    """A Yarra scanner which only operates on VAD regions."""


class YaraScan(common.WinProcessFilter):
    """Scan using yara signatures."""

    __name = "yarascan"

    @classmethod
    def args(cls, parser):
        super(YaraScan, cls).args(parser)
        parser.add_argument("--string", default=None,
                            help="A verbatim string to search for.")

        parser.add_argument("--yara_file", default=None,
                            help="The yara signature file to read.")

        parser.add_argument("--yara_expression", default=None,
                            help="If provided we scan for this yarra "
                            "expression.")

        parser.add_argument("--scan_vads", default=False, action="store_true",
                            help="If specified we scan the vads of "
                            "processes, else we scan their entire address spaces. "
                            "Note that scanning the entire address space will "
                            "typically also include the kernel, while targetting "
                            "VADs only will only include the addresses mapped by "
                            "the process.")

        parser.add_argument("--scan_physical", default=False, action="store_true",
                            help="If specified we scan the physcial address space.")

    def __init__(self, string=None, scan_vads=False, scan_physical=False,
                 yara_file=None, yara_expression=None, **kwargs):
        """Scan using yara signatures.

        Args:
          string: A verbatim string to search for.
          scan_vads: If true we scan the vads of the specified processes, else
            we scan their entire address spaces.
          scan_physical: If true we scan the physical address space.
          yara_file: The yara file to read.
          yara_expression: If provided we scan for this yarra expression.
        """
        super(YaraScan, self).__init__(**kwargs)
        if yara_expression:
            self.rules_source = yara_expression
            self.rules = yara.compile(source=self.rules_source)
        elif string:
            self.rules_source = ('rule r1 {strings: $a = "%s" condition: $a}' %
                                 string)
            self.rules = yara.compile(source=self.rules_source)

        elif yara_file:
            self.rules = yara.compile(yara_file)
        else:
            raise plugin.PluginError("You must specify a yara rule file or "
                                     "string to match.")

        self.scan_vads = scan_vads
        self.scan_physical = scan_physical

    def generate_hits(self, address_space):
        scanner = DiscontigYaraScanner(
            profile=self.profile, session=self.session,
            address_space=address_space,
            rules=self.rules)

        return scanner.scan()

    def render_scan_physical(self, renderer):
        """This method scans the process memory using the VAD."""
        for rule, address, _, hit in self.generate_hits(
            self.physical_address_space):
            renderer.format("Rule: {0}\n", rule)

            context = self.physical_address_space.read(address, 0x40)
            utils.WriteHexdump(renderer, context, base=address)

    def render_kernel_scan(self, renderer):
        modules = self.session.plugins.modules()

        for rule, address, _, hit in self.generate_hits(
            self.kernel_address_space):
            renderer.format("Rule: {0}\n", rule)

            # Find out who owns this hit.
            owner = modules.find_module(address)
            if owner:
                renderer.format("Owner: {0}\n", owner.BaseDllName)
                filename = "kernel.{0:#x}.{1:#x}.dmp".format(
                    owner.obj_offset, address)
            else:
                filename = "kernel.{0:#x}.dmp".format(
                    address)

                renderer.format("Owner: (Unknown Kernel Memory)\n")

            context = self.kernel_address_space.read(address, 0x40)
            utils.WriteHexdump(renderer, context, base=address)

    def render_task_scan(self, renderer, task):
        task_as = task.get_process_address_space()

        for rule, address, _, hit in self.generate_hits(task_as):
            renderer.format("Rule: {0}\n", rule)

            renderer.format("Owner: {0}\n", task.ImageFileName)

            context = task_as.read(address, 0x40)
            utils.WriteHexdump(renderer, context, base=address)

    def render_task_scan_vad(self, renderer, task):
        """This method scans the process memory using the VAD."""
        task_as = task.get_process_address_space()

        scanner = VadYarraScanner(
            session=self.session, rules=self.rules, task=task)

        for rule, address, _, hit in scanner.scan():
            renderer.format("Rule: {0}\n", rule)

            renderer.format("Owner: {0}\n", task.ImageFileName)

            context = task_as.read(address, 0x40)
            utils.WriteHexdump(renderer, context, base=address)


    def render(self, renderer):
        """Render output."""
        if self.scan_physical:
            return self.render_scan_physical(renderer)

        elif self.scan_vads or self.filtering_requested:
            for task in self.filter_processes():
                if self.scan_vads:
                    self.render_task_scan_vad(renderer, task)
                else:
                    self.render_task_scan(renderer, task)

        # We are searching the kernel address space
        else:
            return self.render_kernel_scan(renderer)



# This module provides for a central knowledge base which plugins can use to
# collect information.

import bisect
import re


class SymbolContainer(object):
    """A container class for symbols."""


class SortedAddresses(object):
    """An object to abstract searching for known memory locations."""

    def __init__(self):
        self._data = []
        self._map = {}

    def AddMemoryLocation(self, offset, data):
        """Adds the data to the memory offset."""
        bisect.insort_right(self._data, offset)
        self._map.setdefault(offset, []).append(data)

    def GetSpan(self, offset):
        """Gets the next lowest and next highest data below the offset."""
        idx = bisect.bisect_left(self._data, offset)

        if idx == 0:
            lowest = None
        else:
            lowest = self._map.get(self._data[idx - 1])

        if idx == len(self._data):
            highest = None
        else:
            highest = self._map.get(self._data[idx])

        return lowest, highest


class SymbolAddresses(SortedAddresses):

    def __init__(self):
        super(SymbolAddresses, self).__init__()
        self.symbols = SymbolContainer()

    def AddMemoryLocation(self, offset, func):
        """Add the function object to the list."""
        super(SymbolAddresses, self).AddMemoryLocation(offset, func)

        try:
            module_name, func_name = func.obj_name.split(":")
            module_name = re.match("[_a-zA-Z0-9]+", module_name).group(0)
            func_name = re.match("[_a-zA-Z0-9]+", func_name).group(0)

            module = getattr(self.symbols, module_name, None)
            if module is None:
                module = SymbolContainer()
                setattr(self.symbols, module_name, module)

            setattr(module, func_name, offset)

        except (ValueError, AttributeError):
            pass


# A dict keyed by AS name organizing the symbols known for this address space.
SYMBOLS = {}



# Rekall Memory Forensics
# Copyright (c) 2010, 2011, 2012 Michael Ligh <michael.ligh@mnin.org>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

# pylint: disable=protected-access


from rekall import utils
from rekall.plugins.windows import common

class PsXview(common.WinProcessFilter):
    "Find hidden processes with various process listings"

    __name = "psxview"

    def __init__(self, **kwargs):
        """Find hidden processes with various process listings methods."""
        super(PsXview, self).__init__(**kwargs)

    def check_pslist(self, all_tasks):
        """Enumerate processes from PsActiveProcessHead"""
        return dict((p.obj_vm.vtop(p.obj_offset), p) for p in all_tasks)

    def check_psscan(self):
        """Enumerate processes with pool tag scanning"""
        result = {}
        psscan = self.session.plugins.psscan(session=self.session)
        for hit in psscan.scan_processes():
            result[hit.obj_offset] = hit

        return result

    def check_thrdproc(self):
        """Enumerate processes indirectly by ETHREAD scanning"""
        thrdscan_plugin = self.session.plugins.thrdscan()
        for ethread in thrdscan_plugin.generate_hits():
            if ethread.ExitTime != 0:
                continue

            # Bounce back to the threads owner
            process = ethread.Tcb.m('Process').dereference_as(
                    '_EPROCESS', vm=self.kernel_address_space)

            if not process:
                process = ethread.m('ThreadsProcess').dereference(
                    vm=self.kernel_address_space)

            # Make sure the bounce succeeded
            if (process and process.ExitTime == 0 and
                    process.UniqueProcessId > 0 and
                    process.UniqueProcessId < 0xFFFF):

                yield process.obj_vm.vtop(process.obj_offset)

    def check_pspcid(self):
        """Enumerate processes by walking the PspCidTable"""
        # Follow the pointers to the table base
        PspCidTable = self.kdbg.PspCidTable.cast("Pointer",
                                                 target="_PSP_CID_TABLE")

        # Walk the handle table
        for handle in PspCidTable.handles():
            if handle.get_object_type() == "Process":
                process = handle.dereference_as("_EPROCESS")
                yield process.obj_vm.vtop(process.obj_offset)

    def check_csrss_handles(self):
        """Enumerate processes using the csrss.exe handle table"""
        for task in self.session.plugins.pslist().list_eprocess():
            if utils.SmartUnicode(task.ImageFileName).lower() == "csrss.exe":
                # Gather the handles to process objects
                for handle in task.ObjectTable.handles():
                    if handle.get_object_type() == "Process":
                        process = handle.dereference_as("_EPROCESS")
                        yield process.obj_vm.vtop(process.obj_offset)

    def find_processes(self):
        ps_sources = {}

        pslist_plugin = self.session.plugins.pslist()
        # We store the physical offsets of the _EPROCESS structs.
        ps_sources['pslist'] = set([x.obj_vm.vtop(x.obj_offset)
                                    for x in pslist_plugin.list_eprocess()])

        psscan_plugin = self.session.plugins.psscan(
            address_space=self.physical_address_space)

        ps_sources['psscan'] = set([x.obj_offset
                                    for x in psscan_plugin.scan_processes()])
        ps_sources['thrdproc'] = set(self.check_thrdproc())
        ps_sources['csrss'] = set(self.check_csrss_handles())
        ps_sources['pspcid'] = set(self.check_pspcid())

        # Build a list of offsets from all sources
        all_offsets = set().union(*ps_sources.values())

        for offset in all_offsets:
            # These are the sets that contain this offset
            contains = [k for k, v in ps_sources.items() if offset in v]

            phys_eprocess = self.profile._EPROCESS(
                vm=self.physical_address_space,
                offset=offset)

            virtual_eprocess = psscan_plugin.guess_eprocess_virtual_address(
                phys_eprocess)

            yield phys_eprocess, virtual_eprocess, contains

    def render(self, renderer):
        renderer.table_header(
            [('Offset(P)', 'physical_offset', '[addrpad]'),
             ('Offset(V)', 'virtual_offset', '[addrpad]'),
             ('Name', 'name', '<20'),
             ('PID', 'pid', '>6'),
             ('pslist', 'pslist', '7'),
             ('psscan', 'psscan', '7'),
             ('thrdproc', 'thrdproc', '7'),
             ('pspcdid', 'pspcdid', '7'),
             ('csrss', 'csrss', '7')
             ])

        for process, virtual_eprocess, contains in self.find_processes():
            renderer.table_row(
                process,
                virtual_eprocess,
                process.ImageFileName,
                process.UniqueProcessId,
                ('pslist' in contains),
                ('psscan' in contains),
                ('thrdproc' in contains),
                ('pspcid' in contains),
                ('csrss' in contains),
                )



# Rekall Memory Forensics
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

__author__ = "Michael Cohen <scudette@gmail.com>"

"""Implement OSX support."""

from rekall import obj
from rekall import utils
from rekall.plugins.overlays import basic

darwin_overlay = {
    "proc": [None, {
            "p_list": [None, ["LIST_ENTRY"]],
            "p_sibling": [None, ["LIST_ENTRY"]],

            "p_comm": [None, ["String"]],
            "task": [None, ["Pointer", dict(
                        target="task"
                        )]],
            }],
    "task": [None, {
            "bsd_info": [None, ["Pointer", dict(target="proc")]],
            }],

    "rtentry": [None, {
            "base_calendartime": [None, ["UnixTimeStamp"]],

            "source_ip": lambda x: x.rt_nodes[0].rn_u.rn_leaf.m(
                "rn_Key").dereference_as("sockaddr"),

            "dest_ip": lambda x: x.rt_gateway.deref(),

            "name": lambda x: "%s%s" % (
                x.rt_ifp.if_name.dereference_as("String"),
                x.rt_ifp.if_unit),

            "sent": lambda x: x.m("rt_stats").nstat_txpackets,

            "rx": lambda x: x.rt_expire if x.m("rt_stats") else None,

            "delta": lambda x: (x.rt_expire - x.base_uptime
                                if x.rt_expire else 0),

            }],

    "kmod_info": [None, {
            "name": [None, ["String"]],
            "version": [None, ["String"]],

            # Starting address of the kernel module.
            "address": [None, ["Pointer"]],
            }],

    "sockaddr": [None, {
            # bsd/sys/socket.h: 371
            'sa_family': [None, ["Enumeration", dict(
                        choices={
                            1: "AF_UNIX",
                            2: "AF_INET",
                            18: "AF_LINK",
                            30: "AF_INET6",
                            },
                        target="unsigned char",
                        )]],
            }],

    "sysctl_oid": [None, {
            # xnu-2422.1.72/bsd/sys/sysctl.h: 148
            # This field is reused for two purposes, the first is type of node,
            # while the second is the permissions.
            "oid_kind_type": lambda x: x.m("oid_kind").cast(
                "Enumeration", choices={
                    1: "CTLTYPE_NODE",
                    2: "CTLTYPE_INT",
                    3: "CTLTYPE_STRING",
                    4: "CTLTYPE_QUAD",
                    5: "CTLTYPE_OPAQUE",
                    },
                target="BitField",
                target_args=dict(start_bit=0, end_bit=8),
                ),

            "oid_perms": lambda x: x.m("oid_kind").cast(
                "Flags", maskmap={
                    "CTLFLAG_RD": 0x80000000, # Allow reads of variable */
                    "CTLFLAG_WR": 0x40000000, # Allow writes to the variable
                    "CTLFLAG_LOCKED": 0x00800000, # node will handle locking itself
                    },
                ),

            "oid_name": [None, ["Pointer", dict(target="String")]],

            }],
    "zone": [None, {
            "zone_name": [None, ["Pointer", dict(target="String")]],
            "free_elements": [None, ["Pointer", dict(
                        target="zone_free_element"
                        )]],
            }],

    "vm_map_entry": [None, {
            # xnu-2422.1.72/osfmk/mach/vm_prot.h:81
            "protection": [None, ["Flags", dict(
                        maskmap={
                            "VM_PROT_READ":    1,
                            "VM_PROT_WRITE":   2,
                            "VM_PROT_EXECUTE": 4,
                            },
                        target="BitField",
                        target_args=dict(
                            start_bit=7,
                            end_bit=10
                            )
                        )]],
            }],

    "vnode": [None, {
            "v_name": [None, ["Pointer", dict(target="String")]],

            "path": lambda self: "/".join(reversed(
                    [unicode(y.v_name.deref())
                     for y in self.walk_list("v_parent")])),

            # xnu-2422.1.72/bsd/sys/vnode_internal.h:230
            "v_flag": [None, ["Flags", dict(
                        maskmap={
                            "VROOT":   0x000001,
                            "VTEXT":   0x000002,
                            "VSYSTEM": 0x000004,
                            "VISTTY":  0x000008,
                            "VRAGE":   0x000010,
                            }
                        )]],
            }],

    "ifnet": [None, {
            "if_name": [None, ["Pointer", dict(target="String")]],
            }],

    "session": [None, {
            "s_login": [None, ["String"]],
            }],

    "filedesc": [None, {
            "fd_ofiles": [None, ["Pointer", dict(
                        target="Array",
                        target_args=dict(
                            target="Pointer",
                            count=lambda x: x.fd_lastfile,
                            target_args=dict(
                                target="fileproc"
                                )
                            )
                        )]],
            }],

    "mount": [None, {
            # xnu-2422.1.72/bsd/sys/mount.h
            "mnt_flag": [None, ["Flags", dict(
                        maskmap={
                            "MNT_LOCAL": 0x00001000,
                            "MNT_QUOTA": 0x00002000,
                            "MNT_ROOTFS": 0x00004000,
                            }
                        )]],
            }],

    "vfsstatfs": [None, {
            "f_mntonname": [None, ["String"]],
            "f_mntfromname": [None, ["String"]],
            "f_fstypename": [None, ["String"]],
            }],

    "sockaddr_un": [None, {
            "sun_path": [None, ["String", dict(
                        length=lambda x: x.sun_len,
                        )]],
            }],

    "domain": [None, {
            # xnu-2422.1.72/bsd/sys/domain.h: 99
            'dom_family': [None, ["Enumeration", dict(
                        choices={
                            1: "AF_UNIX",
                            2: "AF_INET",
                            18: "AF_LINK",
                            30: "AF_INET6",
                            },
                        target="int",
                        )]],
            }],

    "tcpcb": [None, {
            "t_state": [None, ["Enumeration", dict(
                        # xnu-2422.1.72/bsd/netinet/tcp_fsm.h: 75
                        choices={
                            0: "TCPS_CLOSED",
                            1: "TCPS_LISTEN",
                            2: "TCPS_SYN_SENT",
                            3: "TCPS_SYN_RECEIVED",
                            4: "TCPS_ESTABLISHED",
                            5: "TCPS_CLOSE_WAIT",
                            6: "TCPS_FIN_WAIT_1",
                            7: "TCPS_CLOSING",
                            8: "TCPS_LAST_ACK",
                            9: "TCPS_FIN_WAIT_2",
                            10: "TCPS_TIME_WAIT",
                            },
                        target="int",
                        )]],
            }],
    "protosw": [None, {
            "pr_protocol": [None, ["Enumeration", dict(
                        # xnu-2422.1.72/bsd/netinet/in.h: 99
                        choices={
                            0: "IPPROTO_IP",
                            1: "IPPROTO_ICMP",
                            2: "IPPROTO_IGMP",
                            4: "IPPROTO_IPV4",
                            6: "IPPROTO_TCP",
                            17: "IPPROTO_UDP",
                            41: "IPPROTO_IPV6",
                            50: "IPPROTO_ESP",
                            51: "IPPROTO_AH",
                            58: "IPPROTO_ICMPV6",
                            255: "IPPROTO_RAW",
                            },
                        target="int",
                        )]],
            }],

    "OSDictionary": [None, {
            "dictionary": [None, ["Pointer", dict(
                        target="Array",
                        target_args=dict(
                            target="dictEntry",
                            count=lambda x: x.count,
                            )
                        )]],
            }],

    "OSString": [None, {
            "value": lambda x: x.obj_profile.UnicodeString(
                offset=x.string, length=x.length),
            }],

    "OSOrderedSet": [None, {
            "array": [None, ["Pointer", dict(
                        target="Array",
                        target_args=dict(
                            target="_Element",
                            count=lambda x: x.count
                            )
                        )]],
            }],

    "PE_state": [None, {
            "bootArgs": [None, ["Pointer", dict(target="boot_args")]],
            }],

    "EfiMemoryRange": [None, {
            # xnu-1699.26.8/pexpert/pexpert/i386/boot.h: 46
            "Type": [None, ["Enumeration", dict(
                        choices={
                            0: "kEfiReservedMemoryType",
                            1: "kEfiLoaderCode",
                            2: "kEfiLoaderData",
                            3: "kEfiBootServicesCode",
                            4: "kEfiBootServicesData",
                            5: "kEfiRuntimeServicesCode",
                            6: "kEfiRuntimeServicesData",
                            7: "kEfiConventionalMemory",
                            8: "kEfiUnusableMemory",
                            9: "kEfiACPIReclaimMemory",
                            10: "kEfiACPIMemoryNVS",
                            11: "kEfiMemoryMappedIO",
                            12: "kEfiMemoryMappedIOPortSpace",
                            13: "kEfiPalCode",
                            14: "kEfiMaxMemoryType",
                            },
                        target="unsigned int"
                        )]],
            }]
    }


darwin64_types = {
    "LIST_ENTRY": [16, {
            "le_next": [0, ["Pointer"]],
            "le_prev": [8, ["Pointer", dict(target="Pointer")]],
            }],
    }


class LIST_ENTRY(obj.Struct):
    """XNU defines lists inline using an annonymous struct. This makes it hard
    for us to automatically support lists because the debugging symbols dont
    indicate this inner struct is of any particular type( since its annonymous).

    We therefore depend on the overlays to redefine each list memeber as a
    LIST_ENTRY member. For example we see code like:

    struct proc {
       LIST_ENTRY(proc) p_list;
       ...

    Where:

    #define  LIST_ENTRY(type)                                       \
      struct {                                                      \
      struct type *le_next;  /* next element */                     \
      struct type **le_prev; /* address of previous next element */ \
    }
    """
    _forward = "le_next"
    _backward = "le_prev"

    def is_valid(self):
        """Must have both valid next and prev pointers."""
        return (self.m(self._forward).is_valid() and
                self.m(self._backward).is_valid())

    def _GetNextEntry(self, type, member):
        return self.m(self._forward).dereference_as(type).m(member)

    def _GetPreviousEntry(self):
        return self.m(self._backward).dereference_as(self.obj_type)

    def dereference_as(self, type, member, vm=None):
        """Recasts the list entry as a member in a type, and return the type.

        Args:
        type: The name of this Struct type.
        member: The name of the member of this Struct.
        address_space: An optional address space to switch during
        deferencing.
        """
        offset = self.obj_profile.get_obj_offset(type, member)

        item = self.obj_profile.Object(
            type_name=type, offset=self.obj_offset - offset,
            vm=vm or self.obj_vm, parent=self.obj_parent,
            name=type, context=self.obj_context)

        return item

    def find_all_lists(self, type, member, seen=None):
        """Follows all the list entries starting from lst.

        We basically convert the list to a tree and recursively search it for
        new nodes. From each node we follow the Flink and then the Blink. When
        we see a node we already have, we backtrack.
        """
        if seen is None:
            seen = set()

        if not self.is_valid():
            return seen

        elif self in seen:
            return seen

        seen.add(self)

        Flink = self._GetNextEntry(type, member)
        Flink.find_all_lists(type, member, seen=seen)

        # Blink = self._GetPreviousEntry()
        # Blink.find_all_lists(type, member, seen=seen)

        return seen

    def list_of_type(self, type, member=None, include_current=True):
        # We sort here to ensure we have stable ordering as the output of this
        # call.
        result = sorted(self.find_all_lists(type, member),
                        key=lambda x: x.obj_offset)

        if member is None:
            member = self.obj_name

        # Return ourselves as the first item.
        if include_current:
            yield self.dereference_as(type, member)

        # We traverse all the _LIST_ENTRYs we can find, and cast them all back
        # to the required member.
        for lst in result:
            # Skip ourselves in this (list_of_type is usually invoked on a list
            # head).
            if lst.obj_offset == self.obj_offset:
                continue

            task = lst.dereference_as(type, member)
            if task:
                # Only yield valid objects (In case of dangling links).
                yield task

    def reflect(self, vm=None):
        """Reflect this list element by following its Flink and Blink.

        This is basically the same as Flink.Blink except that it also checks
        Blink.Flink. It also ensures that Flink and Blink are dereferences to
        the correct type in case the vtypes do not specify them as pointers.

        Returns:
        the result of Flink.Blink.
        """
        result1 = self.m(self._forward).dereference_as(
            self.obj_type, vm=vm).m(self._backward).deref().cast(
            self.obj_type)

        if not result1:
            return obj.NoneObject("Flink not valid.")

        result2 = self.Blink.deref().dereference_as(
            self.obj_type, vm=vm).m(
            self._forward).dereference_as(self.obj_type)

        if result1 != result2:
            return obj.NoneObject("Flink and Blink not consistent.")

        return result1

    def __nonzero__(self):
        ## List entries are valid when both Flinks and Blink are valid
        return bool(self.m(self._forward)) or bool(self.m(self._backward))

    def __iter__(self):
        return self.list_of_type(self.obj_parent.obj_type, self.obj_name)


class queue_entry(LIST_ENTRY):
    _forward = "next"
    _backward = "prev"


class sockaddr_dl(obj.Struct):
    def __str__(self):
        result = []
        for i in range(self.sdl_alen):
            result.append(
                "%.02X" % ord(self.sdl_data[self.sdl_nlen + i].v()))

        return ":".join(result)


class socket(obj.Struct):
    # /xnu-2422.1.72/bsd/kern/socket_info.c: fill_socketinfo()
    def fill_socketinfo(self):
        """Returns a dict instead of a proper socket_info struct."""
        result = utils.AttributeDict(
            domain=self.so_proto.pr_domain.dom_family,
            type=self.so_proto.pr_type,
            protocol=self.so_proto.pr_protocol,
            )

        if result.domain == "AF_INET":
            inp = self.so_pcb.dereference_as("inpcb")
            result.state = inp.inp_ppcb.dereference_as("tcpcb").t_state

            result.remote_port = utils.ntoh(inp.inp_fport)
            result.remote_ip = utils.FormatIPAddress(
                result.domain,
                inp.inp_dependfaddr.inp46_foreign.ia46_addr4.s_addr)

            result.local_port = utils.ntoh(inp.inp_lport)
            result.local_ip = utils.FormatIPAddress(
                result.domain,
                inp.inp_dependladdr.inp46_local.ia46_addr4.s_addr)

        elif result.domain == "AF_INET6":
            inp = self.so_pcb.dereference_as("inpcb")
            result.state = inp.inp_ppcb.dereference_as("tcpcb").t_state

            result.remote_port = utils.ntoh(inp.inp_fport)
            result.remote_ip = utils.FormatIPAddress(
                result.domain,
                inp.inp_dependfaddr.inp6_foreign.m("__u6_addr"))

            result.local_port = utils.ntoh(inp.inp_lport)
            result.local_ip = utils.FormatIPAddress(
                result.domain,
                inp.inp_dependladdr.inp6_local.m("__u6_addr"))

        return result


class sockaddr(obj.Struct):
    def _get_address_obj(self):
        addr = obj.NoneObject("Unknown socket family")

        if self.sa_family == "AF_INET":
            addr = self.cast("sockaddr_in").sin_addr.s_addr

        elif self.sa_family == "AF_INET6":
            addr = self.cast("sockaddr_in6").sin6_addr.m("__u6_addr")

        elif self.sa_family == "AF_LINK":
            addr = self.cast("sockaddr_dl")

        return addr

    def __str__(self):
        result = ""
        addr = self._get_address_obj()
        if addr:
            if self.sa_family in ("AF_INET6", "AF_INET"):
                result = utils.FormatIPAddress(self.sa_family, addr)

            elif self.sa_family == "AF_LINK":
                result = addr

        return str(result)


class vm_map_entry(obj.Struct):
    def find_vnode_object(self):
        """Find the underlying vnode object for the given vm_map_entry.

        xnu-2422.1.72/osfmk/vm/bsd_vm.c: 1339.
        """
        if not self.is_sub_map.v():
            #/*
            #* The last object in the shadow chain has the
            #* relevant pager information.
            #*/

            top_object = self.object.vm_object

            if top_object:
                object = top_object
                while object.shadow:
                    object = object.shadow

                if (object and not object.internal.v() and
                    object.pager_ready.v() and
                    not object.terminating.v() and
                    object.alive.v()):
                    memory_object = object.pager
                    pager_ops = memory_object.mo_pager_ops

                    # If this object points to the vnode_pager_ops, then we
                    # found what we're looking for.  Otherwise, this
                    # vm_map_entry doesn't have an underlying vnode and so we
                    # fall through to the bottom and return NULL.

                    if pager_ops == self.obj_profile.get_constant(
                        "_vnode_pager_ops"):
                        return object.pager.dereference_as(
                            "vnode_pager").vnode_handle

        return obj.NoneObject("VNode not found")


class proc(obj.Struct):
    def get_process_address_space(self):
        cr3 = self.task.map.pmap.pm_cr3
        as_class = self.obj_vm.__class__
        if self.task.map.pmap.pm_task_map == "TASK_MAP_64BIT_SHARED":
            as_class = amd64.AMD64PagedMemory

        return as_class(base=self.obj_vm.base, session=self.obj_vm.session,
                        dtb=cr3, name="Pid %s" % self.p_pid)

    @property
    def argv(self):
        result = []
        array = self.obj_profile.ListArray(
            target="String",
            offset=self.user_stack-self.p_argslen,
            vm=self.get_process_address_space(),
            maximum_size=self.p_argslen,
            )

        for item in array:
            item = unicode(item)

            # The argv array may have null padding for alignment. Discard these
            # empty strings.
            if not len(item):
                continue

            # Total size of the argv array is specified in argc (not counting
            # padding).
            if len(result) > self.p_argc:
                break

            result.append(item)

        # argv[0] is often repeated as the executable name, to avoid confusion,
        # we just discard it.
        if len(result) > 1 and result[0] == result[1]:
            result.pop(0)

        return result


class vnode(obj.Struct):
    @property
    def full_path(self):
        result = []
        vnode = self

        # Iterate here until we hit the root of the filesystem.
        while not vnode.v_flag.VROOT or not vnode.v_mount.mnt_flag.MNT_ROOTFS:
            result.append(vnode.v_name.deref())

            # If there is no parent skip to the mount point.
            vnode = vnode.v_parent or vnode.v_mount.mnt_vnodecovered

        return "/" + "/".join((unicode(x) for x in reversed(result) if x))


class OSDictionary(obj.Struct):
    """The OSDictionary is a general purpose associative array described:

    xnu-1699.26.8/libkern/libkern/c++/OSDictionary.h
    """
    def items(self, value_class=None):
        """Iterate over the associative array and yield key, value pairs."""
        for entry in self.dictionary:
            key = entry.key.dereference_as("OSString").value
            if value_class:
                yield key, entry.value.dereference_as(value_class)
            else:
                yield key, entry.value


class OSOrderedSet(obj.Struct):
    """An OSOrderedSet is a list of OSObject instances.

    xnu-1699.26.8/libkern/libkern/c++/OSOrderedSet.h
    """
    def list_of_type(self, type_name):
        for item in self.array:
            yield item.obj.dereference_as(type_name)


class Darwin32(basic.Profile32Bits, basic.BasicWindowsClasses):
    """A Darwin profile."""
    _md_os = "darwin"
    _md_memory_model = "32bit"
    _md_type = "Kernel"

    def __init__(self, **kwargs):
        super(Darwin32, self).__init__(**kwargs)
        self.add_classes(dict(
                LIST_ENTRY=LIST_ENTRY, queue_entry=queue_entry,
                sockaddr=sockaddr, sockaddr_dl=sockaddr_dl,
                vm_map_entry=vm_map_entry, proc=proc, vnode=vnode,
                socket=socket, OSDictionary=OSDictionary,
                OSOrderedSet=OSOrderedSet,
                ))
        self.add_overlay(darwin_overlay)
        self.add_constants(default_text_encoding="utf8")

    def get_constant_cpp_object(self, constant, **kwargs):
        """A variant of get_constant_object which accounts for name mangling."""
        for key in self.constants:
            if constant in key:
                return self.get_constant_object(key, **kwargs)


class Darwin64(basic.ProfileLP64, Darwin32):
    """Support for 64 bit darwin systems."""

    _md_memory_model = "64bit"

    def __init__(self, **kwargs):
        super(Darwin64, self).__init__(**kwargs)

        self.add_types(darwin64_types)

    def get_constant(self, name, is_address=True):
        """Gets the constant from the profile, correcting for KASLR."""
        base_constant = super(Darwin64, self).get_constant(name)
        if is_address and isinstance(base_constant, (int, long)):
            return base_constant + self.session.GetParameter(
                "vm_kernel_slide", 0)

        return base_constant



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
"""Plugins for inspecting memory allocation zones."""

__author__ = "Michael Cohen <scudette@google.com>"


from rekall import obj
from rekall.plugins.darwin import common


class DarwinListZones(common.DarwinPlugin):
    """List all the allocation zones."""

    __name = "list_zones"

    def ListZones(self):
        first_zone = self.profile.get_constant_object(
            "_first_zone",
            target="Pointer",
            target_args=dict(
                target="zone"
                )
            )

        return first_zone.walk_list("next_zone")

    def GetZone(self, name):
        for zone in self.ListZones():
            if zone.zone_name.deref() == name:
                return zone

        return obj.NoneObject("Zone for %s not found." % name, log=True)

    def render(self, renderer):
        renderer.table_header([
                ("Name", "name", "30"),
                ("Active", "active", ">10"),
                ("Free", "free", ">10"),
                ("Size", "size", ">10")])

        for zone in self.ListZones():
            renderer.table_row(zone.zone_name.deref(),
                               zone.count,
                               zone.m("sum_count") - zone.count,
                               zone.elem_size)


class DarwinDeadProcesses(common.DarwinPlugin):
    """Show deallocated processes which still exist in the zone allocator."""

    __name = "dead_procs"

    def render(self, renderer):
        # Find the proc zone from the allocator.
        proc_zone = DarwinListZones(session=self.session).GetZone("proc")

        # Walk over the free list and get all proc objects.
        procs = []
        for obj in proc_zone.free_elements.walk_list("next"):
            procs.append(obj.cast("proc"))

        if procs:
            # Just delegate the rendering to the regular pslist plugin.
            pslist_plugin = self.session.plugins.pslist(proc=procs)
            pslist_plugin.render(renderer)



# Rekall Memory Forensics
# Copyright (C) 2012
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@gmail.com> based on code by
# {npetroni,awalters}@4tphi.net (Nick Petroni and AAron Walters)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

""" An AS for processing crash dumps """
import logging

from rekall import addrspace
from rekall import obj
from rekall.plugins.addrspaces import standard
from rekall.plugins.overlays.windows import windows

PAGE_SHIFT = 12



class WindowsCrashDumpSpace32(addrspace.RunBasedAddressSpace):
    """ This AS supports windows Crash Dump format """
    order = 30

    PAGE_SIZE = 0x1000

    _md_image = True

    def __init__(self, **kwargs):
        super(WindowsCrashDumpSpace32, self).__init__(**kwargs)
        self.runs = []
        self.offset = 0
        self.fname = ''

        # Check the file for sanity.
        self.check_file()

        # This is a lookup table: (virtual_address, physical_address, length)
        self.runs = []
        file_offset = self.header.size()

        for run in self.header.PhysicalMemoryBlockBuffer.Run:
            self.runs.append((int(run.BasePage) * self.PAGE_SIZE,
                              file_offset,
                              int(run.PageCount) * self.PAGE_SIZE))

            file_offset += run.PageCount * self.PAGE_SIZE

        self.session.StoreParameter(
            "dtb", int(self.header.DirectoryTableBase))

        self.session.StoreParameter(
            "kdbg", int(self.header.KdDebuggerDataBlock))

    def check_file(self):
        """Checks the base file handle for sanity."""

        self.as_assert(self.base,
                       "Must stack on another address space")

        ## Must start with the magic PAGEDUMP
        self.as_assert((self.base.read(0, 8) == 'PAGEDUMP'),
                       "Header signature invalid")

        self.profile = windows.CrashDump32Profile()
        self.header = self.profile.Object(
            "_DMP_HEADER", offset=self.offset, vm=self.base)

        if self.header.DumpType != "Full Dump":
            raise IOError("This is not a full memory crash dump. "
                          "Kernel crash dumps are not supported.")

    def write(self, vaddr, buf):
        # Support writes straddling page runs.
        while len(buf):
            file_offset, available_length = self._get_available_buffer(vaddr, len(buf))
            if file_offset is None:
                raise IOError("Unable to write unmapped runs yet.")

            self.base.write(baddr, buf[:available_length])
            buf = buf[available_length:]


class WindowsCrashDumpSpace64(WindowsCrashDumpSpace32):
    """This AS supports windows Crash Dump format."""
    order = 30

    def check_file(self):
        """Check specifically for 64 bit crash dumps."""

        ## Must start with the magic PAGEDU64
        self.as_assert((self.base.read(0, 8) == 'PAGEDU64'),
                       "Header signature invalid")

        self.profile = windows.CrashDump64Profile()
        self.as_assert(self.profile.has_type("_DMP_HEADER64"),
                       "_DMP_HEADER64 not available in profile")
        self.header = self.profile.Object("_DMP_HEADER64",
                                          offset=self.offset, vm=self.base)

        # The following error is fatal - abort the voting mechanism.

        # Unfortunately trunk rekall does not set this field correctly.
        if self.header.DumpType != "Full Dump":
            logging.warning("This is not a full memory crash dump. "
                            "Kernel crash dumps are not supported.")



# Rekall Memory Forensics
# Copyright (C) 2007,2008 Volatile Systems
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from rekall import obj
from rekall import scan
from rekall import utils

from rekall.plugins.windows import common
from rekall.plugins.windows.gui import win32k_core


class PoolScanWind(common.PoolScanner):
    """PoolScanner for window station objects"""

    def __init__(self, **kwargs):
        super(PoolScanWind, self).__init__(**kwargs)

        self.checks = [('PoolTagCheck', dict(
                    tag=self.profile.get_constant("PoolTag_WindowStation"))),

                       ('CheckPoolSize',  dict(
                    min_size=self.profile.get_obj_size("tagWINDOWSTATION"))),

                       # only look in non-paged or free pools
                       ('CheckPoolType', dict(paged=False, non_paged=True,
                                              free=True)),
                       ('CheckPoolIndex', dict(value = 0)),
                       ]


class WndScan(common.PoolScannerPlugin):
    """Pool scanner for tagWINDOWSTATION (window stations)"""

    __name = "wndscan"

    allocation = ['_POOL_HEADER', '_OBJECT_HEADER', 'tagWINDOWSTATION']

    def __init__(self, **kwargs):
        super(WndScan, self).__init__(**kwargs)
        self.profile = win32k_core.Win32GUIProfile(self.profile)

    def generate_hits(self):
        sessions_plugin = self.session.plugins.sessions()

        scanner = PoolScanWind(profile=self.profile, session=self.session,
                               address_space=self.address_space)

        for pool_obj in scanner.scan():
            window_station = pool_obj.get_object("tagWINDOWSTATION",
                                                 self.allocation)

            # Basic sanity checks are included here
            if not window_station.is_valid():
                continue

            # Find an address space for this window station's session
            session_space = sessions_plugin.find_session_space(
                window_station.dwSessionId)

            if not session_space:
                continue

            # Traverse the tagWINDOWSTATION list in the session address space.
            for winsta in window_station.traverse(session_space.obj_vm):
                if winsta.is_valid():
                    yield winsta, session_space.obj_vm

    def render(self, renderer):
        seen = []
        for window_station, session_address_space in self.generate_hits():
            # Always store the physical addresses to prevent duplicates.
            offset = window_station.obj_vm.vtop(window_station.obj_offset)
            if offset in seen:
                continue

            seen.append(offset)
            renderer.section()

            renderer.format("WindowStation: {0:#x}, Name: {1}, Next: {2:#x}\n",
                            offset,
                            window_station.Name.v(vm=session_address_space),
                            window_station.rpwinstaNext)

            renderer.format("SessionId: {0}, AtomTable: {1:#x}, "
                            "Interactive: {2}\n",
                            window_station.dwSessionId,
                            window_station.pGlobalAtomTable,
                            window_station.Interactive)

            renderer.format(
                "Desktops: {0:L}\n",
                [desk.Name.v(vm=session_address_space)
                 for desk in window_station.desktops(vm=session_address_space)])

            ethread = window_station.ptiDrawingClipboard.pEThread.deref(
                vm=session_address_space)

            renderer.format(
                "ptiDrawingClipboard: pid {0} tid {1}\n",
                ethread.Cid.UniqueProcess, ethread.Cid.UniqueThread)

            last_registered_viewer = window_station.LastRegisteredViewer.deref(
                vm=session_address_space)

            renderer.format("spwndClipOpen: {0:#x}, spwndClipViewer: {1:#x} "
                            "{2} {3}\n",
                            window_station.spwndClipOpen,
                            window_station.spwndClipViewer,
                            last_registered_viewer.UniqueProcessId,
                            last_registered_viewer.ImageFileName)

            renderer.format("cNumClipFormats: {0}, iClipSerialNumber: {1}\n",
                            window_station.cNumClipFormats,
                            window_station.iClipSerialNumber)

            renderer.format(
                "pClipBase: {0:#x}, Formats: {1:L}\n",
                window_station.pClipBase,
                [clip.fmt for clip in window_station.pClipBase.dereference()])



from rekall.plugins.windows import common
from rekall.plugins.windows import connections
from rekall.plugins.windows import connscan
from rekall.plugins.windows import crashinfo
try:
    from rekall.plugins.windows import disassembler
except ImportError:
    pass

from rekall.plugins.windows import filescan
from rekall.plugins.windows import gui
from rekall.plugins.windows import handles
#from rekall.plugins.windows import hibinfo

from rekall.plugins.windows import kdbgscan
from rekall.plugins.windows import kpcr

from rekall.plugins.windows import malware
from rekall.plugins.windows import modscan
from rekall.plugins.windows import modules
from rekall.plugins.windows import netscan
#from rekall.plugins.windows import patcher
from rekall.plugins.windows import pas2kas
from rekall.plugins.windows import pfn
from rekall.plugins.windows import procdump
from rekall.plugins.windows import procinfo
from rekall.plugins.windows import pstree
from rekall.plugins.windows import registry
#from rekall.plugins.windows import sockscan
#from rekall.plugins.windows import ssdt
from rekall.plugins.windows import taskmods
from rekall.plugins.windows import vadinfo



#!/usr/bin/env python
#
#       fileobjscan.py
#       Copyright 2009 Andreas Schuster <a.schuster@yendor.net>
#       Copyright (C) 2009-2011 Volatile Systems
#
#       This program is free software; you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation; either version 2 of the License, or
#       (at your option) any later version.
#
#       This program is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with this program; if not, write to the Free Software
#       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#       MA 02110-1301, USA.

# pylint: disable=protected-access

"""
@author:       Andreas Schuster
@license:      GNU General Public License 2.0 or later
@contact:      a.schuster@forensikblog.de
@organization: http://computer.forensikblog.de/en/
"""

from rekall import obj
from rekall import scan
from rekall.plugins.windows import common


class PoolScanFile(common.PoolScanner):
    """PoolScanner for File objects"""
    def __init__(self, **kwargs):
        super(PoolScanFile, self).__init__(**kwargs)
        self.checks = [
            ('PoolTagCheck', dict(
                    tag=self.profile.get_constant("FILE_POOLTAG"))),

            ('CheckPoolSize', dict(
                    min_size=self.profile.get_obj_size("_FILE_OBJECT"))),

            ('CheckPoolType', dict(
                    paged=True, non_paged=True, free=True)),

            ('CheckPoolIndex', dict(value=0)),
            ]


class FileScan(common.PoolScannerPlugin):
    """ Scan Physical memory for _FILE_OBJECT pool allocations
    """
    __name = "filescan"

    allocation = ['_POOL_HEADER', '_OBJECT_HEADER', '_FILE_OBJECT']

    def generate_hits(self):
        """Generate possible hits."""
        scanner = PoolScanFile(profile=self.profile, session=self.session,
                               address_space=self.address_space)

        for pool_obj in scanner.scan():
            object_obj = pool_obj.get_object("_OBJECT_HEADER", self.allocation)

            if object_obj.get_object_type(self.kernel_address_space) != "File":
                continue

            ## If the string is not reachable we skip it
            file_obj = pool_obj.get_object("_FILE_OBJECT", self.allocation)
            if not file_obj.FileName.v(vm=self.kernel_address_space):
                continue

            yield (object_obj, file_obj)

    def render(self, renderer):
        """Print the output in a table."""

        renderer.table_header([('Offset', "offset_p", '[addrpad]'),
                               ('#Ptr', "ptr_count", '>3'),
                               ('#Hnd', "hnd_count", '>3'),
                               ('Access', "access", '6'),
                               ('Owner', "owner", '[addrpad]'),
                               ('Owner Pid', "owner_pid", '>4'),
                               ('Owner Name', "owner_name", '16'),
                               ('Name', "path", '')
                               ])

        for object_obj, file_obj in self.generate_hits():
            # The Process member in the object_obj sometimes points at the
            # _EPROCESS.
            try:
                # TODO: Currently this only works in Windows 7. Fix for XP.
                owner_process = object_obj.HandleInfo.SingleEntry.Process.dereference(
                    vm=self.kernel_address_space)
            except AttributeError:
                owner_process = obj.NoneObject("HandleInfo not found")

            renderer.table_row(
                file_obj.obj_offset,
                object_obj.PointerCount,
                object_obj.HandleCount,
                file_obj.AccessString,
                owner_process.obj_offset,
                owner_process.UniqueProcessId,
                owner_process.ImageFileName,
                file_obj.FileName.v(vm=self.kernel_address_space))


class PoolScanDriver(PoolScanFile):
    """ Scanner for _DRIVER_OBJECT """

    def __init__(self, **kwargs):
        super(PoolScanDriver, self).__init__(**kwargs)
        self.checks = [
            ('PoolTagCheck', dict(
                    tag=self.profile.get_constant("DRIVER_POOLTAG"))),

            # Must be large enough to hold the driver object.
            ('CheckPoolSize', dict(
                    condition=lambda x: x > self.profile.get_obj_size(
                        "_DRIVER_OBJECT"))),

            ('CheckPoolType', dict(
                    paged=True, non_paged=True, free=True)),

            ('CheckPoolIndex', dict(value=0)),
            ]

class DriverScan(FileScan):
    "Scan for driver objects _DRIVER_OBJECT "

    __name = "driverscan"


    allocation = ['_POOL_HEADER', '_OBJECT_HEADER', '_DRIVER_OBJECT',
                  '_DRIVER_EXTENSION']

    def generate_hits(self):
        """Generate possible hits."""
        scanner = PoolScanDriver(session=self.session,
                                 profile=self.profile,
                                 address_space=self.address_space)

        for pool_obj in scanner.scan():
            object_obj = pool_obj.get_object("_OBJECT_HEADER", self.allocation)
            if object_obj.get_object_type(
                self.kernel_address_space) != "Driver":
                continue

            object_name = object_obj.NameInfo.Name.v(
                vm=self.kernel_address_space)

            driver_obj = pool_obj.get_object("_DRIVER_OBJECT", self.allocation)
            extension_obj = pool_obj.get_object(
                "_DRIVER_EXTENSION", self.allocation)

            yield (object_obj, driver_obj, extension_obj, object_name)


    def render(self, renderer):
        """Renders the text-based output"""
        renderer.table_header([('Offset(P)', "offset_p", '[addrpad]'),
                               ('#Ptr', "ptr_count", '>4'),
                               ('#Hnd', "hnd_count", '>4'),
                               ('Start', "driver_start", '[addrpad]'),
                               ('Size', "driver_size", '[addr]'),
                               ('Service Key', "driver_servicekey", '20'),
                               ('Name', "driver_name", '12'),
                               ('Driver Name', "path", '')
                               ])

        for _ in self.generate_hits():
            object_obj, driver_obj, extension_obj, object_name = _
            renderer.table_row(
                driver_obj.obj_offset,
                object_obj.PointerCount,
                object_obj.HandleCount,
                driver_obj.DriverStart,
                driver_obj.DriverSize,
                extension_obj.ServiceKeyName.v(vm=self.kernel_address_space),
                object_name,
                driver_obj.DriverName.v(vm=self.kernel_address_space))


class PoolScanSymlink(PoolScanFile):
    """ Scanner for symbolic link objects """
    def __init__(self, **kwargs):
        super(PoolScanSymlink, self).__init__(**kwargs)
        self.checks = [
            ('PoolTagCheck', dict(
                    tag=self.profile.get_constant("SYMLINK_POOLTAG"))),

            ('CheckPoolSize', dict(
                    min_size=self.profile.get_obj_size(
                        "_OBJECT_SYMBOLIC_LINK"))),

            ('CheckPoolType', dict(paged=True, non_paged=True, free=True)),
            ]


class SymLinkScan(FileScan):
    "Scan for symbolic link objects "

    __name = "symlinkscan"

    allocation = ['_POOL_HEADER', '_OBJECT_HEADER', '_OBJECT_SYMBOLIC_LINK']

    def generate_hits(self):
        """Generate possible hits."""
        scanner = PoolScanSymlink(profile=self.profile, session=self.session,
                                  address_space=self.address_space)
        for pool_obj in scanner.scan():
            object_obj = pool_obj.get_object("_OBJECT_HEADER", self.allocation)
            if object_obj.get_object_type(
                self.kernel_address_space) != "SymbolicLink":
                continue

            object_name = object_obj.NameInfo.Name.v(
                vm=self.kernel_address_space)

            link_obj = pool_obj.get_object(
                "_OBJECT_SYMBOLIC_LINK", self.allocation)

            yield object_obj, link_obj, object_name

    def render(self, renderer):
        """ Renders text-based output """
        renderer.table_header([('Offset(P)', "offset_p", '[addrpad]'),
                               ('#Ptr', "ptr_count", '>6'),
                               ('#Hnd', "hnd_count", '>6'),
                               ('Creation time', "symlink_creation_time", '24'),
                               ('From', "symlink_from", ''),
                               ('To', "symlink_to", '60'),
                               ])


        for o, link, name in self.generate_hits():
            renderer.table_row(
                link.obj_offset,
                o.PointerCount,
                o.HandleCount,
                link.CreationTime or '',
                name,
                link.LinkTarget.v(vm=self.kernel_address_space))


class PoolScanMutant(PoolScanDriver):
    """ Scanner for Mutants _KMUTANT """
    def __init__(self, **kwargs):
        super(PoolScanMutant, self).__init__(**kwargs)
        self.checks = [
            ('PoolTagCheck', dict(tag=self.profile.get_constant(
                        "MUTANT_POOLTAG"))),

            ('CheckPoolSize', dict(
                    min_size=self.profile.get_obj_size("_KMUTANT"))),

            ('CheckPoolType', dict(
                    paged=True, non_paged=True, free=True)),

            ('CheckPoolIndex', dict(value=0)),
            ]


class MutantScan(FileScan):
    "Scan for mutant objects _KMUTANT "

    __name = "mutantscan"

    allocation = ['_POOL_HEADER', '_OBJECT_HEADER', '_KMUTANT']

    def __init__(self, silent=False, **kwargs):
        """Scan for mutant objects _KMUTANT.

        Args:
           silent: Suppress less meaningful results.
        """
        super(MutantScan, self).__init__(**kwargs)
        self.silent = silent

    def generate_hits(self):
        scanner = PoolScanMutant(profile=self.profile, session=self.session,
                                 address_space=self.address_space)

        for pool_obj in scanner.scan():
            object_obj = pool_obj.get_object("_OBJECT_HEADER", self.allocation)
            if object_obj.get_object_type(
                self.kernel_address_space) != "Mutant":
                continue

            object_name = object_obj.NameInfo.Name.v(
                vm=self.kernel_address_space)

            if self.silent and not object_name:
                continue

            mutant = pool_obj.get_object("_KMUTANT", self.allocation)
            yield (object_obj, mutant, object_name)

    def render(self, renderer):
        """Renders the output"""

        renderer.table_header([('Offset(P)', "offset_p", '[addrpad]'),
                               ('#Ptr', "ptr_count", '>4'),
                               ('#Hnd', "hnd_count", '>4'),
                               ('Signal', "mutant_signal", '4'),
                               ('Thread', "mutant_thread", '[addrpad]'),
                               ('CID', "cid", '>9'),
                               ('Name', "mutant_name", '')
                               ])

        for object_obj, mutant, _ in self.generate_hits():
            if mutant.OwnerThread > 0x80000000:
                thread = self.profile._ETHREAD(
                    offset=mutant.OwnerThread, vm=self.kernel_address_space)

                CID = "{0}:{1}".format(thread.Cid.UniqueProcess,
                                       thread.Cid.UniqueThread)
            else:
                CID = ""

            renderer.table_row(
                mutant.obj_offset,
                object_obj.PointerCount,
                object_obj.HandleCount,
                mutant.Header.SignalState,
                mutant.OwnerThread,
                CID,
                object_obj.NameInfo.Name.v(vm=self.kernel_address_space))


class CheckProcess(scan.ScannerCheck):
    """ Check sanity of _EPROCESS """
    kernel = 0x80000000

    def check(self, found):
        """Check a possible _EPROCESS."""
        ## The offset of the object is determined by subtracting the offset
        ## of the PoolTag member to get the start of Pool Object. This done
        ## because PoolScanners search for the PoolTag.
        pool_base = found - self.profile.get_obj_offset(
            '_POOL_HEADER', 'PoolTag')

        pool_obj = self.profile._POOL_HEADER(offset=pool_base,
                                             vm=self.address_space)

        pool_align = self.profile.constants['PoolAlignment']

        eprocess = self.profile._EPROCESS(
            vm=self.address_space,
            offset=(pool_base + pool_obj.BlockSize * pool_align -
                    self.profile.get_obj_size("_EPROCESS"))
            )

        if eprocess.Pcb.DirectoryTableBase == 0:
            return False

        if eprocess.Pcb.DirectoryTableBase % 0x20 != 0:
            return False

        list_head = eprocess.ThreadListHead

        if (list_head.Flink < self.kernel) or (list_head.Blink < self.kernel):
            return False

        return True


class PoolScanProcess(common.PoolScanner):
    """PoolScanner for File objects"""
    allocation = ['_POOL_HEADER', '_OBJECT_HEADER', '_EPROCESS']

    def __init__(self, **kwargs):
        super(PoolScanProcess, self).__init__(**kwargs)
        self.checks = [
            # Must have the right pool tag.
            ('PoolTagCheck', dict(
                    tag=self.profile.get_constant("EPROCESS_POOLTAG"))),

            # Must be large enough for an _EPROCESS.
            ('CheckPoolSize', dict(min_size=self.profile.get_obj_size(
                        "_EPROCESS"))),

            ('CheckPoolType', dict(
                    paged=True, non_paged=True, free=True)),

            ('CheckPoolIndex', dict(value=0)),
            ]

    def scan(self, **_):
        for pool_obj in super(PoolScanProcess, self).scan():
            eprocess = pool_obj.get_object("_EPROCESS", self.allocation)

            if eprocess.Pcb.DirectoryTableBase == 0:
                continue

            if eprocess.Pcb.DirectoryTableBase % 0x20 != 0:
                continue

            yield eprocess


class PSScan(common.PoolScannerPlugin):
    """Scan Physical memory for _EPROCESS pool allocations."""

    __name = "psscan"

    def __init__(self, **kwargs):
        """Scan Physical memory for _EPROCESS pool allocations.

        Status flags:
          E: A known _EPROCESS address from pslist.
          P: A known pid from pslist.
        """
        super(PSScan, self).__init__(**kwargs)

    def guess_eprocess_virtual_address(self, eprocess):
        """Try to guess the virtual address of the eprocess."""
        # This is the list entry of the ProcessListEntry reflected through the
        # next process in the list
        list_entry = eprocess.ThreadListHead.Flink.dereference_as(
            '_LIST_ENTRY', vm=self.kernel_address_space).Blink.dereference()

        # Take us back to the _EPROCESS offset
        list_entry_offset = self.profile.get_obj_offset(
            '_EPROCESS', 'ThreadListHead')

        # The virtual eprocess should be the same as the physical one
        kernel_eprocess_offset = list_entry.obj_offset - list_entry_offset

        return kernel_eprocess_offset

    def scan_processes(self):
        """Generate possible hits."""
        # Just grab the AS and scan it using our scanner
        scanner = PoolScanProcess(session=self.session,
                                  profile=self.profile,
                                  address_space=self.address_space)

        return scanner.scan()

    def render(self, renderer):
        """Render results in a table."""
        # Try to do a regular process listing so we can compare if the process
        # is known.
        pslist = self.session.plugins.pslist()

        # These are virtual addresses.
        known_eprocess = set()
        known_pids = set()
        for task in pslist.list_eprocess():
            known_eprocess.add(task.obj_offset)
            known_pids.add(task.UniqueProcessId)

        renderer.table_header([('Offset', "offset_p", '[addrpad]'),
                               ('Offset(V)', "offset_v", '[addrpad]'),
                               ('Name', "file_name", '16'),
                               ('PID', "pid", '>6'),
                               ('PPID', "ppid", '>6'),
                               ('PDB', "pdb", '[addrpad]'),
                               ('Stat', 'stat', "4"),
                               ('Time created', "process_create_time", '24'),
                               ('Time exited', "process_exit_time", '24')]
                               )

        for eprocess in self.scan_processes():

            # Try to guess the virtual address of the eprocess
            eprocess_virtual_address = self.guess_eprocess_virtual_address(
                eprocess)

            known = ""
            if eprocess_virtual_address in known_eprocess:
                known += "E"

            if eprocess.UniqueProcessId in known_pids:
                known += "P"

            renderer.table_row(eprocess.obj_offset,
                               eprocess_virtual_address,
                               eprocess.ImageFileName,
                               eprocess.UniqueProcessId,
                               eprocess.InheritedFromUniqueProcessId,
                               eprocess.Pcb.DirectoryTableBase,
                               known,
                               eprocess.CreateTime or '',
                               eprocess.ExitTime or '')



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

"""
@author:       Andrew Case
@license:      GNU General Public License 2.0 or later
@contact:      atcuno@gmail.com
@organization: Digital Forensics Solutions
"""
import os

from rekall import testlib
from rekall.plugins import core
from rekall.plugins.linux import common


class LinuxPsList(common.LinProcessFilter):
    """Gathers active tasks by walking the task_struct->task list.

    It does not display the swapper process. If the DTB column is blank, the
    item is likely a kernel thread.
    """
    __name = "pslist"

    def render(self, renderer):
    	renderer.table_header( [("Offset (V)", "offset_v", "[addrpad]"),
                                ("Name", "file_name", "20s"),
                                ("PID", "pid", ">6"),
                                ("PPID", "ppid", ">6"),
                                ("UID", "uid", ">6"),
                                ("GID", "gid", ">6"),
                                ("DTB", "dtb", "[addrpad]"),
                                ("Start Time", "start_time", ">24"),
                                ])

        for task in self.filter_processes():
            start_time = (task.start_time.as_timestamp()+
                          task.start_time.getboottime())

            dtb = self.kernel_address_space.vtop(task.mm.pgd)
            renderer.table_row(task.obj_offset,
                               task.comm,
                               task.pid,
                               task.parent.pid,
                               task.uid,
                               task.gid,
                               dtb, start_time)


class LinMemMap(common.LinProcessFilter):
    """Dumps the memory map for linux tasks."""

    __name = "memmap"

    @classmethod
    def args(cls, parser):
        """Declare the command line args we need."""
        super(LinMemMap, cls).args(parser)
        parser.add_argument(
            "--coalesce", default=False, action="store_true",
            help="Merge contiguous pages into larger ranges.")

    def __init__(self, coalesce=False, **kwargs):
        """Calculates the memory regions mapped by a process.

        Args:
          coalesce: Merge pages which are contiguous in memory into larger
             ranges.
        """
        self.coalesce = coalesce
        super(LinMemMap, self).__init__(**kwargs)

    def render(self, renderer):
        for task in self.filter_processes():
            renderer.section()
            renderer.RenderProgress("Dumping pid {0}".format(
                    task.pid))

            task_space = task.get_process_address_space()
            renderer.format(u"Process: '{0}' pid: {1:6}\n",
                            task.comm, task.pid)

            if not task_space:
                renderer.write("Unable to read pages for task.\n")
                continue

            renderer.table_header([("Virtual", "offset_v", "[addrpad]"),
                                   ("Physical", "offset_p", "[addrpad]"),
                                   ("Size", "process_size", "[addr]")])

            if self.coalesce:
                ranges = task_space.get_address_ranges()
            else:
                ranges = task_space.get_available_addresses()

            for virtual_address, length in ranges:
                phys_address = task_space.vtop(virtual_address)
                renderer.table_row(virtual_address, phys_address, length)


class LinMemDump(core.DirectoryDumperMixin, LinMemMap):
    """Dump the addressable memory for a process.

    This plugin traverses the page tables and dumps all accessible memory for
    the task. Note that this excludes kernel memory even though it is mapped
    into the task.
    """

    __name = "memdump"

    def dump_process(self, task, fd):
        task_as = task.get_process_address_space()

        # We want to stop dumping memory when we reach the max addressable
        # memory by the process (anything above that is kernel memory).
        max_memory = task.mm.task_size

        result = []
        for virtual_address, length in task_as.get_address_ranges(
            end=max_memory):
            phys_address = task_as.vtop(virtual_address)
            result.append((fd.tell(), length, virtual_address))
            fd.write(self.physical_address_space.read(phys_address, length))

        return result

    def write_index(self, renderer, maps, fd):
        old_file_addr = old_length = old_virtual = 0
        for file_addr, length, virtual in maps:
            # Merge the addresses as much as possible.
            if (old_virtual + old_length == virtual and
                old_file_addr + old_length == file_addr):
                old_length += length
                continue

            renderer.table_row(old_file_addr, old_length, old_virtual)

            old_file_addr = file_addr
            old_length = length
            old_virtual = virtual

        if old_file_addr != file_addr:
            renderer.table_row(old_file_addr, old_length, old_virtual)

    def render(self, renderer):
        if self.dump_dir is None:
            raise plugin.PluginError("Dump directory not specified.")

        for task in self.filter_processes():
            filename = os.path.join(
                self.dump_dir, u"{0}_{1:d}.dmp".format(task.comm, task.pid))

            renderer.write(u"Writing {0} {1:6x} to {2}\n".format(
                    task.comm, task, filename))

            with open(filename, 'wb') as fd:
                maps = self.dump_process(task, fd)

            with open(filename + ".idx", 'wb') as fd:
                temp_renderer = renderer.classes["TextRenderer"](fd=fd)
                temp_renderer.table_header([
                        ("File Address", "file_addr", "[addrpad]"),
                        ("Length", "length", "[addrpad]"),
                        ("Virtual Addr", "virtual", "[addrpad]")])

                self.write_index(temp_renderer, maps, fd)

                temp_renderer.flush()


class TestLinMemDump(testlib.HashChecker):
    PARAMETERS = dict(
        commandline="memdump --proc_regex bash --dump-dir %(tempdir)s"
        )

# We only care about PIDTYPE_PID here.
# http://lxr.free-electrons.com/source/include/linux/pid.h?v=3.8#L6
#enum pid_type
# {
#         PIDTYPE_PID,
# };
PIDTYPE_PID = 0


class PidHashTable(LinuxPsList):
    """List processes by enumerating the pid hash tables."""

    __name = "pidhashtable"

    def list_tasks(self):
        # According to
        # http://lxr.free-electrons.com/source/kernel/pid.c?v=3.8#L566, the
        # pid_hash table is a pointer to a dynamically allocated array of
        # hlist_head.
        pidhash_shift = self.profile.get_constant_object(
            "pidhash_shift", "unsigned int")

        pidhash = self.profile.get_constant_object(
            "pid_hash",
            target="Pointer",
            target_args=dict(
                target="Array",
                target_args=dict(
                    count=1 << pidhash_shift,
                    target="hlist_head")
                )
            )

        seen = set()

        # Now we iterate over all the hash slots in the hash table to retrieve
        # their struct upid entries.
        for hash in pidhash:
            for upid in hash.list_of_type("upid", "pid_chain"):
                # upid structures are contained inside pid structures:
                # http://lxr.free-electrons.com/source/kernel/pid.c?v=3.8#L351
                # container_of(pnr, struct pid, numbers[ns->level]);
                level = upid.ns.level

                pid = self.profile.pid(
                    upid.obj_offset -
                    self.profile.get_obj_offset("pid", "numbers") -
                    level * self.profile.get_obj_size("pid"))

                # Here we only care about regular PIDs.
                for task in pid.tasks[PIDTYPE_PID].list_of_type(
                    "task_struct", "pids"):
                    if task not in seen:
                        yield task
                        seen.add(task)



# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@gmail.com>
# Based on code by Aaron Walters.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

import os

from rekall import plugin
from rekall.plugins.windows import common
from rekall.plugins.addrspaces import crash
from rekall.plugins.addrspaces import standard
from rekall.plugins.overlays.windows import windows


class CrashInfo(common.AbstractWindowsCommandPlugin):
    """Dump crash-dump information"""

    __name = "crashinfo"

    @classmethod
    def is_active(cls, config):
        """We are only active if the profile is windows."""
        return isinstance(
            config.physical_address_space, crash.WindowsCrashDumpSpace32)

    def render(self, renderer):
        """Renders the crashdump header as text"""
        if not isinstance(
            self.physical_address_space, crash.WindowsCrashDumpSpace32):
            raise plugin.PluginError("Image is not a windows crash dump.")

        renderer.write(self.physical_address_space.header)

        renderer.table_header(
            [("FileOffset", "file_offset", "[addrpad]"),
             ("Start Address", "file_start_address", "[addrpad]"),
             ("Length", "file_length", "[addr]")])
        page_size = self.physical_address_space.PAGE_SIZE
        for start, file_offset, count in self.physical_address_space.runs:
            renderer.table_row(file_offset,
                               start * page_size,
                               count * page_size)


class Raw2Dump(common.WindowsCommandPlugin):
    """Convert the physical address space to a crash dump."""

    __name = "raw2dmp"

    def __init__(self, destination=None, overwrite=False,
                 buffer_size=10*1024*1024, **kwargs):
        """Convert the physical address space to a crash dump.

        Args:
          destination: The destination path to write the crash dump.
          overwrite: Should the output be overwritten?
        """
        super(Raw2Dump, self).__init__(**kwargs)
        self.profile = windows.CrashDump64Profile()

        self.buffer_size = buffer_size
        self.destination = destination
        if not destination:
            raise plugin.PluginError("A destination must be provided.")

        if not overwrite and os.access(destination, os.F_OK):
            raise plugin.PluginError(
                "Unable to overwrite the destination file '%s'" % destination)

    def render(self, renderer):
        PAGE_SIZE = 0x1000

        # We write the image to the destination using the WriteableAddressSpace.
        out_as = standard.WriteableAddressSpace(filename=self.destination)

        if self.profile.metadata("memory_model") == "64bit":
            header = self.profile.Object('_DMP_HEADER64', offset=0, vm=out_as)

            # Pad the header area with PAGE pattern:
            out_as.write(0, "PAGE" * (header.size() / 4))

            header.KdDebuggerDataBlock = int(self.kdbg) | 0xFFFF000000000000
            out_as.write(4, "DU64")
        else:
            header = self.profile.Object('_DMP_HEADER64', offset=0, vm=out_as)
            header.KdDebuggerDataBlock = int(self.kdbg)

            # Pad the header area with PAGE pattern:
            out_as.write(0, "PAGE" * (header.size() / 4))
            out_as.write(4, "DUMP")

            # PEA address spaces.
            if getattr(self.kernel_address_space, "pae", None):
                header.PaeEnabled = 1

        # Scanning the memory region near KDDEBUGGER_DATA64 for
        # DBGKD_GET_VERSION64
        dbgkd = self.kdbg.dbgkd_version64()

        # Write the runs from our physical address space.
        number_of_pages = 0
        i = None

        for i, (start, length) in enumerate(
            self.physical_address_space.get_available_addresses()):
            # Convert to pages
            start = start / PAGE_SIZE
            length = length / PAGE_SIZE

            header.PhysicalMemoryBlockBuffer.Run[i].BasePage = start
            header.PhysicalMemoryBlockBuffer.Run[i].PageCount = length
            number_of_pages += length

        # Must be at least one run.
        if i is None:
            raise plugin.PluginError(
                "Physical address space has no available data.")

        header.PhysicalMemoryBlockBuffer.NumberOfRuns = i + 1
        header.PhysicalMemoryBlockBuffer.NumberOfPages = number_of_pages

        # Set members of the crash header
        header.MajorVersion = dbgkd.MajorVersion.v()
        header.MinorVersion = dbgkd.MinorVersion.v()
        header.DirectoryTableBase = self.session.GetParameter("dtb")
        header.PfnDataBase = self.kdbg.MmPfnDatabase.v()
        header.PsLoadedModuleList = self.kdbg.PsLoadedModuleList.v()
        header.PsActiveProcessHead = self.kdbg.PsActiveProcessHead.v()
        header.MachineImageType = dbgkd.MachineType.v()

        # Find the number of processors
        header.NumberProcessors = len(list(self.kdbg.kpcrs()))

        # Zero out the BugCheck members
        header.BugCheckCode = 0x00000000
        header.BugCheckCodeParameter[0] = 0x00000000
        header.BugCheckCodeParameter[1] = 0x00000000
        header.BugCheckCodeParameter[2] = 0x00000000
        header.BugCheckCodeParameter[3] = 0x00000000

        # Set the sample run information

        header.RequiredDumpSpace = number_of_pages + header.size() / PAGE_SIZE
        header.SystemTime = 0
        header.DumpType = 1

        # Zero out the remaining non-essential fields from ContextRecordOffset
        # to ExceptionOffset.
        out_as.write(header.ContextRecord.obj_offset,
                     "\x00" * (header.m("Exception").obj_offset -
                               header.ContextRecord.obj_offset))

        # Set the "converted" comment
        out_as.write(header.Comment.obj_offset,
                     "File was converted with Rekall Memory Forensics" + "\x00")

        # Now copy the physical address space to the output file.
        output_offset = header.size()
        for _ in self.physical_address_space.get_available_addresses():
            start, length = _

            # Convert to pages
            start = start / PAGE_SIZE
            length = length / PAGE_SIZE

            renderer.write("\nRun [0x%08X, 0x%08X] \n" % (
                    start, length))
            data_length = length * PAGE_SIZE
            start_offset = start * PAGE_SIZE
            offset = 0
            while data_length > 0:
                to_read = min(data_length, self.buffer_size)

                data = self.physical_address_space.read(
                    start_offset + offset, to_read)

                out_as.write(output_offset, data)
                output_offset += len(data)
                offset += len(data)
                data_length -= len(data)
                renderer.RenderProgress(
                    "Wrote %sMB.", (start_offset + offset)/1024/1024)





# Rekall Memory Forensics
#
# Copyright 2013 Google Inc. All Rights Reserved.
#
# Authors:
# Michael Cohen <scudette@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""This plugin is used for displaying information about the Kernel Processor
Control Blocks.
"""

# pylint: disable=protected-access

import logging

from rekall.plugins.windows import common


class KPCR(common.AbstractWindowsCommandPlugin):
    """A plugin to print all KPCR blocks."""
    __name = "kpcr"

    @classmethod
    def is_active(cls, session):
        # Only active for windows 7 right now.
        return (super(KPCR, cls).is_active(session) and
                session.profile.metadata("major") >= 6)

    @classmethod
    def args(cls, parser):
        """Declare the command line args we need."""
        super(KPCR, cls).args(parser)
        parser.add_argument("--eprocess",
                            help="An _EPROCESS virtual address to start "
                            "scanning with.")

    def __init__(self, eprocess=None, **kwargs):
        """Print all KPCR Objects.

        Args:
          eprocess: an _EPROCESS virtual address to use to enumerate threads.
        """
        super(KPCR, self).__init__(**kwargs)
        self.eprocess = eprocess

    def find_kpcr(self, task):
        """Given an _EPROCESS object, find KPCR."""
        # This is the offset of the WaitListHead from the start of the _KPCR
        # struct.
        offset = self.profile._KPCR(vm=None).Prcb.WaitListHead.obj_offset

        seen_threads = set()
        seen = {}

        # Iterate over all the threads of this process.
        for kthread in task.Pcb.ThreadListHead.list_of_type(
            "_KTHREAD", "ThreadListEntry"):

            # Skip threads we already examined.
            if kthread.obj_offset in seen_threads:
                break

            seen_threads.add(kthread.obj_offset)

            # Look for threads in the Wait state. If this thread is in the Wait
            # state, the WaitListEntry will belong to the list of all waiting
            # threads. By following this list we should get to the list head
            # which lives inside the _KPCR object.
            for kwaiter in kthread.WaitListEntry.list_of_type(
                "_KTHREAD", "WaitListEntry"):

                self.session.report_progress()

                if kwaiter.obj_offset in seen_threads:
                    break

                seen_threads.add(kwaiter.obj_offset)

                # Assume the kwaiter is actually the KPRCB.WaitListHead.
                possible_kpcr = self.profile._KPCR(
                    kwaiter.WaitListEntry.obj_offset - offset)

                # Check for validity using the usual condition.
                if possible_kpcr.Self == possible_kpcr.obj_offset:
                    if possible_kpcr.obj_offset not in seen:
                        seen[possible_kpcr.obj_offset] = possible_kpcr


        # Return all the _KPCR structs we know about.
        return seen.values()

    def render(self, renderer):
        eprocess = self.eprocess

        if self.session.system_eprocess:
            # Convert the eprocess to the virtual address space by reflecting
            # through the ActiveProcessLinks.
            eprocess = self.session.system_eprocess.ActiveProcessLinks.reflect(
                vm=self.session.kernel_address_space).dereference_as(
                "_EPROCESS", "ActiveProcessLinks")

        if not eprocess:
            for task in self.session.plugins.pslist().list_eprocess():
                eprocess = task
                break

        if not eprocess:
            logging.error("Require at least one _EPROCESS to use.")
            return

        for kpcr in self.find_kpcr(eprocess):
            renderer.section()

            renderer.table_header([("Property", "property", "<30"),
                                   ("Value", "value", "<")])

            renderer.table_row("Offset (V)", hex(kpcr.obj_offset))
            renderer.table_row(
                "Offset (P)", hex(self.session.kernel_address_space.vtop(
                        kpcr.obj_offset)))

            renderer.table_row("KdVersionBlock", kpcr.KdVersionBlock)

            renderer.table_row("IDT", hex(kpcr._IDT))
            renderer.table_row("GDT", hex(kpcr._GDT))

            current_thread = kpcr.ProcessorBlock.CurrentThread
            idle_thread = kpcr.ProcessorBlock.IdleThread
            next_thread = kpcr.ProcessorBlock.NextThread

            if current_thread:
                renderer.format("{0:<30}: {1:#x} TID {2} ({3}:{4})\n",
                                "CurrentThread",
                                current_thread, current_thread.Cid.UniqueThread,
                                current_thread.owning_process().ImageFileName,
                                current_thread.Cid.UniqueProcess,
                                )

            if idle_thread:
                renderer.format("{0:<30}: {1:#x} TID {2} ({3}:{4})\n",
                                "IdleThread",
                                idle_thread, idle_thread.Cid.UniqueThread,
                                idle_thread.owning_process().ImageFileName,
                                idle_thread.Cid.UniqueProcess,
                                )

            if next_thread:
                renderer.format("{0:<30}: {1:#x} TID {2} ({3}:{4})\n",
                                "NextThread",
                                next_thread,
                                next_thread.Cid.UniqueThread,
                                next_thread.owning_process().ImageFileName,
                                next_thread.Cid.UniqueProcess,
                                )

            renderer.format("{0:<30}: CPU {1} ({2} @ {3} MHz)\n",
                            "Details",
                            kpcr.ProcessorBlock.Number,
                            kpcr.ProcessorBlock.VendorString,
                            kpcr.ProcessorBlock.MHz)

            renderer.format(
                "{0:<30}: {1:#x}\n", "CR3/DTB",
                kpcr.ProcessorBlock.ProcessorState.SpecialRegisters.Cr3)



# Rekall Memory Forensics
# Copyright (C) 2007,2008 Volatile Systems
# Copyright (C) 2010,2011,2012 Michael Hale Ligh <michael.ligh@mnin.org>
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

from rekall import obj
from rekall.plugins.windows import common
from rekall.plugins.windows.gui import win32k_core


class Sessions(common.WinProcessFilter):
    """List details on _MM_SESSION_SPACE (user logon sessions)"""

    __name = "sessions"

    def __init__(self, **kwargs):
        super(Sessions, self).__init__(**kwargs)
        self.profile = win32k_core.Win32GUIProfile(self.profile)

    def session_spaces(self):
        """Generates unique _MM_SESSION_SPACE objects.

        Generates unique _MM_SESSION_SPACE objects referenced by active
        processes.

        Yields:
          _MM_SESSION_SPACE instantiated from the session space's address space.
        """
        seen = []
        for proc in self.filter_processes():
            if proc.SessionId and proc.SessionId.v() not in seen:
                ps_ad = proc.get_process_address_space()
                if ps_ad:
                    seen.append(proc.SessionId.v())

                    yield self.profile._MM_SESSION_SPACE(
                        offset=proc.Session.v(), vm=ps_ad)

    def find_session_space(self, session_id):
        """ Get a _MM_SESSION_SPACE object by its ID.

        Args:
          session_id: the session ID to find.

        Returns:
          _MM_SESSION_SPACE instantiated from the session space's address space.
        """
        for session in self.session_spaces():
            if session.SessionId == session_id:
                return session

        return obj.NoneObject("Cannot locate a session %s", session_id)

    def render(self, renderer):
        # Use the modules plugin to resolve the module addresses.
        module_plugin = self.session.plugins.modules()

        for session in self.session_spaces():
            renderer.section()

            renderer.format("Session(V): {0:x} ID: {1} Processes: {2}\n",
                            session.obj_offset,
                            session.SessionId,
                            len(list(session.processes())))

            renderer.format("PagedPoolStart: {0:x} PagedPoolEnd {1:x}\n",
                            session.PagedPoolStart,
                            session.PagedPoolEnd)

            for process in session.processes():
                renderer.format(" Process: {0} {1} {2} @ {3:#x}\n",
                                process.UniqueProcessId,
                                process.ImageFileName,
                                process.CreateTime,
                                process)

            for image in session.images():
                module = module_plugin.find_module(image.Address)

                renderer.format(" Image: {0:#x}, Address {1:x}, Name: {2}\n",
                                image.obj_offset,
                                image.Address,
                                module.BaseDllName)



#!/usr/bin/python

# Rekall Memory Forensics
# Copyright 2013 Google Inc. All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or (at
# your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#

"""Support IPython 1.0."""

# pylint: disable=protected-access

__author__ = "Michael Cohen <scudette@gmail.com>"

from rekall import config
from rekall import constants
from rekall import utils

try:
    from IPython.terminal import embed
except ImportError:
    try:
        from IPython.frontend.terminal import embed
    except ImportError:
        embed = None

from IPython.config.loader import Config


config.DeclareOption("--ipython_engine",
                     help="IPython engine, e.g. notebook.")


def RekallCompleter(self, text):
    command_parts = self.line_buffer.split(" ")
    command = command_parts[0]

    global_matches = set(self.global_matches(command))

    # Only complete if there is exactly one object which matches and a space was
    # typed after it. e.g.:
    # pslist <cursor>
    if (global_matches and len(global_matches) == 1 and
        len(command_parts) > 1):

        # Get the object and ask it about the list of default args.
        obj = self.namespace.get(global_matches.pop())
        try:
            matches = ["%s=" % x for x in obj.get_default_arguments()]
            return [x for x in matches if x.startswith(text)]
        except Exception:
            pass

    return []


def Shell(user_session):
    # This should bring back the old autocall behaviour. e.g.:
    # In [1]: pslist
    cfg = Config()
    cfg.InteractiveShellEmbed.autocall = 2

    cfg.PromptManager.in_template = (
        r'{color.LightCyan}'
        r'{session.state.profile}:{session.state.base_filename}'
        r'{color.LightBlue}{color.Green} \T> ')

    cfg.PromptManager.in2_template = (
        r'{color.Green}|{color.LightGreen}\D{color.Green}> ')

    cfg.PromptManager.out_template = r'Out<\#> '
    cfg.InteractiveShell.separate_in = ''
    cfg.InteractiveShell.separate_out = ''
    cfg.InteractiveShell.separate_out2 = ''

    shell = embed.InteractiveShellEmbed(
        config=cfg, user_ns=user_session._locals)

    shell.Completer.merge_completions = False
    shell.banner = constants.BANNER
    shell.exit_msg = constants.GetQuote()
    shell.set_custom_completer(RekallCompleter, 0)

    # Do we need to pre-run something?
    if user_session.run is not None:
        execfile(user_session.run, user_session._locals)

    # Workaround for completer bug.
    import IPython.core.completerlib
    IPython.core.completerlib.get_ipython = lambda: shell

    shell(local_ns=user_session._locals)

    return True



def NotebookSupport(_):

    # The following only reveals hidden imports to pyinstaller.
    if False:
        import IPython.html.notebookapp
        import IPython.html.base.handlers
        import IPython.html.tree.handlers
        import IPython.html.auth.login
        import IPython.html.auth.logout
        import IPython.html.notebook.handlers
        import IPython.html.services.kernels.handlers
        import IPython.html.services.notebooks.handlers
        import IPython.html.services.clusters.handlers
        import IPython.kernel.ioloop
        import IPython.kernel.zmq.kernelapp

        import rekall.interactive

        import zmq.backend.cython
        import zmq.eventloop.ioloop

    argv = ["notebook", "-c",
            "from rekall import interactive; "
            "interactive.ImportEnvironment();", "--autocall", "2",
            "--notebook-dir",
            config.GetConfigFile().get("notebook_dir",
                                       config.GetHomeDir())
            ]

    import IPython

    IPython.start_ipython(argv=argv)
    return True
